import { Select } from '@blueprintjs/select';
import { Button, Colors, Intent, MenuItem } from '@blueprintjs/core';
import styled from '@emotion/styled';
import React, { useLayoutEffect, useMemo, useRef } from 'react';
import { IconNames } from '@blueprintjs/icons';
import { scaleLinear, scaleSequential } from 'd3-scale';
import { interpolateRgbBasis } from 'd3-interpolate';
import { ClassNames } from '@emotion/core';
import { COLOR_SCHEMES, DEFAULT_COLOR_SCHEME } from './colors';
import { Row } from './Boxes';
var ColorRampLabel = styled.div({
    width: 55,
    fontSize: '10px',
    textAlign: 'center',
});
var ColorRampCanvas = styled.canvas({
    display: 'flex',
    flexGrow: 1,
    border: "1px solid " + Colors.DARK_GRAY5,
});
var ColorRamp = function (props) {
    var colorScheme = props.colorScheme, _a = props.width, width = _a === void 0 ? 60 : _a, _b = props.height, height = _b === void 0 ? 13 : _b, reverse = props.reverse;
    var colorScale = scaleSequential(interpolateRgbBasis(COLOR_SCHEMES[colorScheme] || DEFAULT_COLOR_SCHEME));
    var canvasRef = useRef(null);
    var xScale = useMemo(function () { return scaleLinear().range([0, width]); }, [width]);
    useLayoutEffect(function () {
        var current = canvasRef.current;
        if (current) {
            var ctx = current.getContext('2d');
            if (ctx) {
                ctx.clearRect(0, 0, width, height);
                for (var i = 0; i < width; i++) {
                    ctx.fillStyle = colorScale(xScale.invert(i));
                    ctx.fillRect(reverse ? width - i - 1 : i, 0, 1, height);
                }
            }
        }
    });
    return (React.createElement(Row, null,
        React.createElement(ColorRampLabel, null, colorScheme),
        React.createElement(ColorRampCanvas, { ref: canvasRef, width: width, height: height })));
};
var ColorSchemeSelector = function (_a) {
    var _b = _a.selected, selected = _b === void 0 ? 'Default' : _b, reverse = _a.reverse, onChange = _a.onChange;
    var itemRenderer = function (colorScheme, _a) {
        var modifiers = _a.modifiers, handleClick = _a.handleClick;
        if (!modifiers.matchesPredicate) {
            return null;
        }
        return (React.createElement(MenuItem, { key: colorScheme, active: modifiers.active, disabled: modifiers.disabled, intent: selected === colorScheme ? Intent.PRIMARY : Intent.NONE, onClick: handleClick, icon: React.createElement(ColorRamp, { colorScheme: colorScheme, reverse: reverse }) }));
    };
    return (React.createElement(ClassNames, null, function (_a) {
        var css = _a.css;
        return (React.createElement(Select, { items: Object.keys(COLOR_SCHEMES), filterable: false, activeItem: selected, itemRenderer: itemRenderer, popoverProps: {
                minimal: true,
                captureDismiss: true,
                usePortal: false,
                fill: true,
                targetClassName: css({
                    '& > div': {
                        flex: '1',
                    },
                }),
            }, onItemSelect: onChange },
            React.createElement(Button, { rightIcon: IconNames.CARET_DOWN, fill: true, icon: React.createElement(ColorRamp, { colorScheme: selected, reverse: reverse }) })));
    }));
};
export default ColorSchemeSelector;
//# sourceMappingURL=ColorSchemeSelector.js.map