var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
import { DeckGL } from '@deck.gl/react';
import { MapController } from '@deck.gl/core';
import * as React from 'react';
import { useCallback, useEffect, useMemo, useReducer, useRef, useState } from 'react';
import { alea } from 'seedrandom';
import { _MapContext as MapContext, StaticMap } from 'react-map-gl';
import FlowMapLayer, { PickingType, } from '@flowmap.gl/core';
import { Button, ButtonGroup, Classes, Colors, HTMLSelect, Intent } from '@blueprintjs/core';
import { getViewStateForLocations, LocationTotalsLegend } from '@flowmap.gl/react';
import WebMercatorViewport from '@math.gl/web-mercator';
import { Absolute, Box, BoxStyle, Column, Description, LegendTitle, Title, TitleBox, ToastContent, } from './Boxes';
import { FlowTooltipContent, formatCount, LocationTooltipContent } from './TooltipContent';
import Tooltip from './Tooltip';
import { Link, useHistory } from 'react-router-dom';
import Collapsible, { Direction } from './Collapsible';
import { ConfigPropName, getFlowDestId, getFlowMagnitude, getFlowOriginId, getLocationCentroid, getLocationId, } from './types';
import Message from './Message';
import LoadingSpinner from './LoadingSpinner';
import NoScrollContainer from './NoScrollContainer';
import styled from '@emotion/styled';
import { IconNames } from '@blueprintjs/icons';
import LocationsSearchBox from './LocationSearchBox';
import Away from './Away';
import { ActionType, getInitialState, HighlightType, mapTransition, MAX_PITCH, MAX_ZOOM_LEVEL, MIN_PITCH, MIN_ZOOM_LEVEL, reducer, stateToQueryString, } from './FlowMap.state';
import { getAvailableClusterZoomLevels, getClusterIndex, getClusterZoom, getDarkMode, getDiffMode, getFetchedFlows, getFlowMagnitudeExtent, getFlowMapColors, getFlowsForFlowMapLayer, getFlowsSheets, getInvalidLocationIds, getLocations, getLocationsForFlowMapLayer, getLocationsForSearchBox, getLocationsHavingFlows, getLocationsInBbox, getLocationsTree, getLocationTotals, getLocationTotalsExtent, getMapboxMapStyle, getMaxLocationCircleSize, getSortedFlowsForKnownLocations, getTimeExtent, getTimeGranularity, getTotalCountsByTime, getTotalFilteredCount, getTotalUnfilteredCount, getUnknownLocations, NUMBER_OF_FLOWS_TO_DISPLAY, } from './FlowMap.selectors';
import { AppToaster } from './AppToaster';
import useDebounced from './hooks';
import SharePopover from './SharePopover';
import SettingsPopover from './SettingsPopover';
import MapDrawingEditor, { MapDrawingMode } from './MapDrawingEditor';
import getBbox from '@turf/bbox';
import booleanPointInPolygon from '@turf/boolean-point-in-polygon';
import Timeline from './Timeline';
import { findAppropriateZoomLevel } from '@flowmap.gl/cluster/dist-esm';
var CONTROLLER_OPTIONS = {
    type: MapController,
    doubleClickZoom: false,
    dragRotate: true,
    touchRotate: true,
    minZoom: MIN_ZOOM_LEVEL,
    maxZoom: MAX_ZOOM_LEVEL,
};
/* This is temporary until mixBlendMode style prop works in <DeckGL> as before v8 */
var DeckGLOuter = styled.div(function (_a) {
    var cursor = _a.cursor, baseMapOpacity = _a.baseMapOpacity, darkMode = _a.darkMode;
    return "\n  & #deckgl-overlay {\n    mix-blend-mode: " + (darkMode ? 'screen' : 'multiply') + ";\n  }\n  & .mapboxgl-map {\n    opacity: " + baseMapOpacity + "\n  }\n  " + (cursor != null ? "& #view-default-view { cursor: " + cursor + "; }" : '') + ",\n";
});
export var ErrorsLocationsBlock = styled.div(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n  font-size: 8px;\n  padding: 10px;\n  max-height: 100px;\n  overflow: auto;\n"], ["\n  font-size: 8px;\n  padding: 10px;\n  max-height: 100px;\n  overflow: auto;\n"])));
var SelectedTimeRangeBox = styled(BoxStyle)(function (props) { return ({
    display: 'flex',
    alignSelf: 'center',
    fontSize: 12,
    padding: '5px 10px',
    borderRadius: 5,
    backgroundColor: props.darkMode ? Colors.DARK_GRAY4 : Colors.LIGHT_GRAY4,
    textAlign: 'center',
}); });
var TimelineBox = styled(BoxStyle)({
    minWidth: 400,
    display: 'block',
    boxShadow: '0 0 5px #aaa',
    borderTop: '1px solid #999',
});
var TotalCount = styled.div(function (props) { return ({
    padding: 5,
    borderRadius: 5,
    backgroundColor: props.darkMode ? Colors.DARK_GRAY4 : Colors.LIGHT_GRAY4,
    textAlign: 'center',
}); });
export var MAX_NUM_OF_IDS_IN_ERROR = 100;
var FlowMap = function (props) {
    var _a, _b;
    var inBrowser = props.inBrowser, embed = props.embed, config = props.config, spreadSheetKey = props.spreadSheetKey, locationsFetch = props.locationsFetch, flowsFetch = props.flowsFetch;
    var deckRef = useRef();
    var history = useHistory();
    var initialState = useMemo(function () { return getInitialState(config, history.location.search); }, [
        config,
    ]);
    var outerRef = useRef(null);
    var _c = __read(useReducer(reducer, initialState), 2), state = _c[0], dispatch = _c[1];
    var _d = __read(useState(false), 2), mapDrawingEnabled = _d[0], setMapDrawingEnabled = _d[1];
    var selectedTimeRange = state.selectedTimeRange;
    var timeGranularity = getTimeGranularity(state, props);
    var timeExtent = getTimeExtent(state, props);
    var totalCountsByTime = getTotalCountsByTime(state, props);
    var totalFilteredCount = getTotalFilteredCount(state, props);
    var totalUnfilteredCount = getTotalUnfilteredCount(state, props);
    useEffect(function () {
        if (timeExtent) {
            if (!selectedTimeRange ||
                // reset selectedTimeRange if not within the timeExtent
                !(timeExtent[0] <= selectedTimeRange[0] && selectedTimeRange[1] <= timeExtent[1])) {
                dispatch({
                    type: ActionType.SET_TIME_RANGE,
                    range: timeExtent,
                });
            }
        }
    }, [timeExtent, selectedTimeRange]);
    var _e = __read(useDebounced(function () {
        if (inBrowser)
            return;
        var locationSearch = "?" + stateToQueryString(state);
        if (locationSearch !== history.location.search) {
            history.replace(__assign(__assign({}, history.location), { search: locationSearch }));
        }
    }, 250, [state, history.location.search]), 1), updateQuerySearch = _e[0];
    useEffect(updateQuerySearch, [history, state]);
    var viewport = state.viewport, tooltip = state.tooltip, animationEnabled = state.animationEnabled, baseMapEnabled = state.baseMapEnabled;
    var allFlows = getFetchedFlows(state, props);
    var allLocations = getLocations(state, props);
    var locationsHavingFlows = getLocationsHavingFlows(state, props);
    var locations = getLocationsForFlowMapLayer(state, props);
    var flows = getFlowsForFlowMapLayer(state, props);
    var flowsSheets = getFlowsSheets(config);
    var handleKeyDown = function (evt) {
        if (evt instanceof KeyboardEvent && evt.key === 'Escape') {
            if (mapDrawingEnabled) {
                setMapDrawingEnabled(false);
            }
            else {
                if (tooltip) {
                    hideTooltip();
                }
                if (state.highlight) {
                    highlight(undefined);
                }
                // dispatch({ type: ActionType.CLEAR_SELECTION });
            }
        }
    };
    useEffect(function () {
        window.addEventListener('keydown', handleKeyDown);
        return function () {
            window.removeEventListener('keydown', handleKeyDown);
        };
    });
    var _f = __read(useState(0), 2), time = _f[0], setTime = _f[1];
    // Use useRef for mutable variables that we want to persist
    // without triggering a re-render on their change
    var requestAnimationFrameRef = useRef();
    var animate = useCallback(function (time) {
        var loopLength = 1800;
        var animationSpeed = 20;
        var loopTime = loopLength / animationSpeed;
        var timestamp = time / 1000;
        setTime(((timestamp % loopTime) / loopTime) * loopLength);
        requestAnimationFrameRef.current = requestAnimationFrame(animate);
    }, [requestAnimationFrameRef, setTime]);
    useEffect(function () {
        if (animationEnabled) {
            requestAnimationFrameRef.current = requestAnimationFrame(animate);
        }
        else {
            var animationFrame = requestAnimationFrameRef.current;
            if (animationFrame != null && animationFrame > 0) {
                window.cancelAnimationFrame(animationFrame);
                requestAnimationFrameRef.current = undefined;
            }
        }
        return function () {
            if (requestAnimationFrameRef.current != null) {
                cancelAnimationFrame(requestAnimationFrameRef.current);
            }
        };
    }, [animationEnabled, animate]);
    var showErrorToast = useCallback(function (errorText) {
        if (config[ConfigPropName.IGNORE_ERRORS] !== 'yes') {
            AppToaster.show({
                intent: Intent.WARNING,
                icon: IconNames.WARNING_SIGN,
                timeout: 0,
                message: React.createElement(ToastContent, null, errorText),
            });
        }
    }, [config]);
    var invalidLocations = getInvalidLocationIds(state, props);
    useEffect(function () {
        if (invalidLocations) {
            showErrorToast(React.createElement(React.Fragment, null,
                "Locations with the following IDs have invalid coordinates:",
                React.createElement(ErrorsLocationsBlock, null,
                    (invalidLocations.length > MAX_NUM_OF_IDS_IN_ERROR
                        ? invalidLocations.slice(0, MAX_NUM_OF_IDS_IN_ERROR)
                        : invalidLocations)
                        .map(function (id) { return "" + id; })
                        .join(', '),
                    invalidLocations.length > MAX_NUM_OF_IDS_IN_ERROR &&
                        "\u2026 and " + (invalidLocations.length - MAX_NUM_OF_IDS_IN_ERROR) + " others"),
                "Make sure you named the columns \"lat\" and \"lon\" and didn't confuse latitudes and longitudes. The coordinates must be in decimal form. If your coordinates are in degrees-minutes-seconds (DSM format) you can convert them with",
                ' ',
                React.createElement(Away, { href: "https://www.latlong.net/degrees-minutes-seconds-to-decimal-degrees" }, "this tool"),
                ' ',
                "for example."));
        }
    }, [invalidLocations, showErrorToast]);
    var unknownLocations = getUnknownLocations(state, props);
    var flowsForKnownLocations = getSortedFlowsForKnownLocations(state, props);
    useEffect(function () {
        if (unknownLocations) {
            if (flowsForKnownLocations && allFlows) {
                var ids = Array.from(unknownLocations).sort();
                showErrorToast(React.createElement(React.Fragment, null,
                    "Locations with the following IDs couldn't be found in the locations sheet:",
                    React.createElement(ErrorsLocationsBlock, null,
                        (ids.length > MAX_NUM_OF_IDS_IN_ERROR ? ids.slice(0, MAX_NUM_OF_IDS_IN_ERROR) : ids)
                            .map(function (id) { return "" + id; })
                            .join(', '),
                        ids.length > MAX_NUM_OF_IDS_IN_ERROR &&
                            "\u2026 and " + (ids.length - MAX_NUM_OF_IDS_IN_ERROR) + " others"),
                    formatCount(allFlows.length - flowsForKnownLocations.length),
                    " flows were omitted.",
                    flowsForKnownLocations.length === 0 && (React.createElement("div", { style: { marginTop: '1em' } },
                        "Make sure the columns are named header row in the flows sheet is correct. There must be ",
                        React.createElement("b", null, "origin"),
                        ", ",
                        React.createElement("b", null, "dest"),
                        ", and ",
                        React.createElement("b", null, "count"),
                        "."))));
            }
        }
    }, [unknownLocations, showErrorToast, allFlows, flowsForKnownLocations]);
    var adjustViewportToLocations = state.adjustViewportToLocations;
    useEffect(function () {
        if (!adjustViewportToLocations) {
            return;
        }
        var width = window.innerWidth;
        var height = window.innerHeight;
        if (allLocations != null) {
            var draft = getViewStateForLocations(locationsHavingFlows !== null && locationsHavingFlows !== void 0 ? locationsHavingFlows : allLocations, getLocationCentroid, [width, height], { pad: 0.1 });
            if (!draft.zoom) {
                draft = {
                    zoom: 1,
                    latitude: 0,
                    longitude: 0,
                };
            }
            dispatch({
                type: ActionType.SET_VIEWPORT,
                viewport: __assign(__assign(__assign({ width: width,
                    height: height }, draft), { minZoom: MIN_ZOOM_LEVEL, maxZoom: MAX_ZOOM_LEVEL, minPitch: MIN_PITCH, maxPitch: MAX_PITCH, bearing: 0, pitch: 0, altitude: 1.5 }), mapTransition(1000)),
                adjustViewportToLocations: false,
            });
        }
    }, [allLocations, locationsHavingFlows, adjustViewportToLocations]);
    var clusterIndex = getClusterIndex(state, props);
    var handleChangeClusteringAuto = function (value) {
        if (!value) {
            if (clusterIndex) {
                var availableZoomLevels = clusterIndex.availableZoomLevels;
                if (availableZoomLevels != null) {
                    dispatch({
                        type: ActionType.SET_MANUAL_CLUSTER_ZOOM,
                        manualClusterZoom: findAppropriateZoomLevel(clusterIndex.availableZoomLevels, viewport.zoom),
                    });
                }
            }
        }
        dispatch({
            type: ActionType.SET_CLUSTERING_AUTO,
            clusteringAuto: value,
        });
    };
    var _g = __read(useState(embed && document.fullscreenEnabled), 2), showFullscreenButton = _g[0], setShowFullscreenButton = _g[1];
    useEffect(function () {
        function handleFullScreenChange() {
            setShowFullscreenButton(embed && document.fullscreenEnabled && !document.fullscreenElement);
        }
        document.addEventListener('fullscreenchange', handleFullScreenChange);
        return function () { return document.removeEventListener('fullscreenchange', handleFullScreenChange); };
    }, [setShowFullscreenButton]);
    var getContainerClientRect = useCallback(function () {
        var container = outerRef.current;
        if (!container)
            return undefined;
        return container.getBoundingClientRect();
    }, [outerRef]);
    var getMercator = useCallback(function () {
        var containerBounds = getContainerClientRect();
        if (!containerBounds)
            return undefined;
        var width = containerBounds.width, height = containerBounds.height;
        return new WebMercatorViewport(__assign(__assign({}, viewport), { width: width,
            height: height }));
    }, [viewport, getContainerClientRect]);
    var showTooltip = function (bounds, content) {
        var containerBounds = getContainerClientRect();
        if (!containerBounds)
            return;
        var top = containerBounds.top, left = containerBounds.left;
        dispatch({
            type: ActionType.SET_TOOLTIP,
            tooltip: {
                target: __assign(__assign({}, bounds), { left: left + bounds.left, top: top + bounds.top }),
                placement: 'top',
                content: content,
            },
        });
    };
    var highlight = function (highlight) {
        dispatch({ type: ActionType.SET_HIGHLIGHT, highlight: highlight });
    };
    var _h = __read(useDebounced(showTooltip, 500), 2), showTooltipDebounced = _h[0], cancelShowTooltipDebounced = _h[1];
    var _j = __read(useDebounced(highlight, 500), 2), highlightDebounced = _j[0], cancelHighlightDebounced = _j[1];
    var hideTooltip = function () {
        dispatch({
            type: ActionType.SET_TOOLTIP,
            tooltip: undefined,
        });
        cancelShowTooltipDebounced();
    };
    var showFlowTooltip = function (pos, info) {
        var _a = __read(pos, 2), x = _a[0], y = _a[1];
        var r = 5;
        var bounds = {
            left: x - r,
            top: y - r,
            width: r * 2,
            height: r * 2,
        };
        var content = (React.createElement(FlowTooltipContent, { flow: info.object, origin: info.origin, dest: info.dest, config: config }));
        if (state.tooltip) {
            showTooltip(bounds, content);
            cancelShowTooltipDebounced();
        }
        else {
            showTooltipDebounced(bounds, content);
        }
    };
    var showLocationTooltip = function (info) {
        var location = info.object, circleRadius = info.circleRadius;
        var mercator = getMercator();
        if (!mercator)
            return;
        var _a = __read(mercator.project(getLocationCentroid(location)), 2), x = _a[0], y = _a[1];
        var r = circleRadius + 5;
        var selectedLocations = state.selectedLocations;
        var bounds = {
            left: x - r,
            top: y - r,
            width: r * 2,
            height: r * 2,
        };
        var content = (React.createElement(LocationTooltipContent, { locationInfo: info, isSelectionEmpty: !selectedLocations, isSelected: selectedLocations && selectedLocations.find(function (id) { return id === location.id; }) ? true : false, config: config }));
        if (state.tooltip) {
            showTooltip(bounds, content);
            cancelShowTooltipDebounced();
        }
        else {
            showTooltipDebounced(bounds, content);
        }
    };
    var handleHover = function (info) {
        var type = info.type, object = info.object, x = info.x, y = info.y;
        switch (type) {
            case PickingType.FLOW: {
                if (object) {
                    highlight({
                        type: HighlightType.FLOW,
                        flow: object,
                    });
                    cancelHighlightDebounced();
                    showFlowTooltip([x, y], info);
                }
                else {
                    highlight(undefined);
                    cancelHighlightDebounced();
                    hideTooltip();
                }
                break;
            }
            case PickingType.LOCATION: {
                if (object) {
                    highlightDebounced({
                        type: HighlightType.LOCATION,
                        locationId: getLocationId(object),
                    });
                    showLocationTooltip(info);
                }
                else {
                    highlight(undefined);
                    cancelHighlightDebounced();
                    hideTooltip();
                }
                break;
            }
            default: {
                highlight(undefined);
                cancelHighlightDebounced();
                hideTooltip();
            }
        }
    };
    if (locationsFetch.loading) {
        return React.createElement(LoadingSpinner, null);
    }
    if (locationsFetch.error || flowsFetch.error) {
        return (React.createElement(Message, null, spreadSheetKey
            ? React.createElement(React.Fragment, null,
                React.createElement("p", null,
                    "Oops\u2026 Couldn't fetch data from", " ",
                    React.createElement("a", { href: "https://docs.google.com/spreadsheets/d/" + spreadSheetKey }, "this spreadsheet"),
                    ".", " "),
                React.createElement("p", null,
                    "If you are the owner of this spreadsheet, make sure you have shared it by doing the following:",
                    React.createElement("ol", null,
                        React.createElement("li", null, "Click the \u201CShare\u201D button in the spreadsheet"),
                        React.createElement("li", null, "Change the selection from \u201CRestricted\u201D to \u201CAnyone with the link\u201D in the drop-down under \u201CGet link\u201D"))))
            : React.createElement("p", null, "Oops\u2026 Couldn't fetch data")));
    }
    var searchBoxLocations = getLocationsForSearchBox(state, props);
    var title = config[ConfigPropName.TITLE];
    var description = config[ConfigPropName.DESCRIPTION];
    var sourceUrl = config[ConfigPropName.SOURCE_URL];
    var sourceName = config[ConfigPropName.SOURCE_NAME];
    var authorUrl = config[ConfigPropName.AUTHOR_URL];
    var authorName = config[ConfigPropName.AUTHOR_NAME];
    var mapboxAccessToken = config[ConfigPropName.MAPBOX_ACCESS_TOKEN];
    var diffMode = getDiffMode(state, props);
    var darkMode = getDarkMode(state, props);
    var mapboxMapStyle = getMapboxMapStyle(state, props);
    var getHighlightForZoom = function () {
        var highlight = state.highlight, clusteringEnabled = state.clusteringEnabled;
        if (!highlight || !clusteringEnabled) {
            return highlight;
        }
        var clusterTree = getClusterIndex(state, props);
        var clusterZoom = getClusterZoom(state, props);
        if (!clusterTree || clusterZoom === undefined) {
            return undefined;
        }
        var isValidForClusterZoom = function (itemId) {
            var cluster = clusterTree.getClusterById(itemId);
            if (cluster) {
                return cluster.zoom === clusterZoom;
            }
            else {
                var minZoom = clusterTree.getMinZoomForLocation(itemId);
                if (minZoom === undefined || clusterZoom >= minZoom) {
                    return true;
                }
            }
            return false;
        };
        switch (highlight.type) {
            case HighlightType.LOCATION:
                var locationId = highlight.locationId;
                return isValidForClusterZoom(locationId) ? highlight : undefined;
            case HighlightType.FLOW:
                var _a = highlight.flow, origin_1 = _a.origin, dest = _a.dest;
                if (isValidForClusterZoom(origin_1) && isValidForClusterZoom(dest)) {
                    return highlight;
                }
                return undefined;
        }
        return undefined;
    };
    var handleClick = function (info, event) {
        switch (info.type) {
            case PickingType.LOCATION:
            // fall through
            case PickingType.LOCATION_AREA: {
                var object = info.object;
                if (object) {
                    dispatch({
                        type: ActionType.SELECT_LOCATION,
                        locationId: getLocationId(object),
                        incremental: event.srcEvent.shiftKey,
                    });
                }
                break;
            }
        }
    };
    var handleChangeSelectLocations = function (selectedLocations) {
        dispatch({
            type: ActionType.SET_SELECTED_LOCATIONS,
            selectedLocations: selectedLocations,
        });
    };
    var handleChangeLocationFilterMode = function (mode) {
        dispatch({
            type: ActionType.SET_LOCATION_FILTER_MODE,
            mode: mode,
        });
    };
    var handleViewStateChange = function (_a) {
        var viewState = _a.viewState;
        dispatch({
            type: ActionType.SET_VIEWPORT,
            viewport: viewState,
        });
    };
    var locationsTree = getLocationsTree(state, props);
    var handleTimeRangeChanged = function (range) {
        dispatch({
            type: ActionType.SET_TIME_RANGE,
            range: range,
        });
    };
    var handleMapFeatureDrawn = function (feature) {
        if (feature != null) {
            var bbox = getBbox(feature);
            var candidates = getLocationsInBbox(locationsTree, bbox);
            if (candidates) {
                var inPolygon = candidates.filter(function (loc) {
                    return booleanPointInPolygon(getLocationCentroid(loc), feature);
                });
                if (inPolygon.length > 0) {
                    handleChangeSelectLocations(inPolygon.map(getLocationId));
                    // TODO: support incremental
                }
                else {
                    handleChangeSelectLocations(undefined);
                }
            }
        }
        setMapDrawingEnabled(false);
        if (deckRef.current && deckRef.current.deck) {
            // This is a workaround for a deck.gl issue.
            // Without it the following happens:
            // 1. When finishing a map drawing and releasing the mouse over a deck.gl layer
            //    an onClick event is generated.
            // 2. Deck.gl sets the info of this event to _lastPointerDownInfo
            //    which holds the last object that was clicked any time before
            //    starting the map drawing.
            // 3. The object info is passed to the onClick handler of the corresponding
            //    layer which leads to selecting the object and altering the map drawing selection.
            deckRef.current.deck._lastPointerDownInfo = null;
        }
    };
    var handleToggleMapDrawing = function () {
        setMapDrawingEnabled(!mapDrawingEnabled);
    };
    var handleZoomIn = function () {
        dispatch({ type: ActionType.ZOOM_IN });
    };
    var handleZoomOut = function () {
        dispatch({ type: ActionType.ZOOM_OUT });
    };
    var handleResetBearingPitch = function () {
        dispatch({ type: ActionType.RESET_BEARING_PITCH });
    };
    var handleSelectFlowsSheet = function (event) {
        var sheet = event.currentTarget.value;
        var onSetFlowsSheet = props.onSetFlowsSheet;
        if (onSetFlowsSheet) {
            onSetFlowsSheet(sheet);
            handleChangeSelectLocations(undefined);
        }
    };
    var handleFullScreen = function () {
        var outer = outerRef.current;
        if (outer) {
            if (outer.requestFullscreen) {
                outer.requestFullscreen();
            }
            else if (outer.webkitRequestFullscreen) {
                outer.webkitRequestFullscreen();
            }
        }
    };
    var getLayers = function () {
        var animationEnabled = state.animationEnabled, adaptiveScalesEnabled = state.adaptiveScalesEnabled, locationTotalsEnabled = state.locationTotalsEnabled, darkMode = state.darkMode, colorSchemeKey = state.colorSchemeKey, fadeAmount = state.fadeAmount;
        var layers = [];
        if (locations && flows) {
            var id = [
                'flow-map',
                animationEnabled ? 'animated' : 'arrows',
                locationTotalsEnabled ? 'withTotals' : '',
                colorSchemeKey,
                darkMode ? 'dark' : 'light',
                fadeAmount,
            ].join('-');
            var locationTotals_1 = getLocationTotals(state, props);
            var highlight_1 = getHighlightForZoom();
            layers.push(new FlowMapLayer(__assign(__assign(__assign(__assign({ id: id, animate: animationEnabled, animationCurrentTime: time, diffMode: getDiffMode(state, props), colors: getFlowMapColors(state, props), locations: locations,
                flows: flows, showOnlyTopFlows: NUMBER_OF_FLOWS_TO_DISPLAY, getLocationCentroid: getLocationCentroid,
                getFlowMagnitude: getFlowMagnitude,
                getFlowOriginId: getFlowOriginId,
                getFlowDestId: getFlowDestId,
                getLocationId: getLocationId, getLocationTotalIn: function (loc) { var _a; return ((_a = locationTotals_1 === null || locationTotals_1 === void 0 ? void 0 : locationTotals_1.get(loc.id)) === null || _a === void 0 ? void 0 : _a.incoming) || 0; }, getLocationTotalOut: function (loc) { var _a; return ((_a = locationTotals_1 === null || locationTotals_1 === void 0 ? void 0 : locationTotals_1.get(loc.id)) === null || _a === void 0 ? void 0 : _a.outgoing) || 0; }, getLocationTotalWithin: function (loc) { var _a; return ((_a = locationTotals_1 === null || locationTotals_1 === void 0 ? void 0 : locationTotals_1.get(loc.id)) === null || _a === void 0 ? void 0 : _a.within) || 0; }, getAnimatedFlowLineStaggering: function (d) {
                    // @ts-ignore
                    return new alea(d.origin + "-" + d.dest)();
                }, showTotals: true, maxLocationCircleSize: getMaxLocationCircleSize(state, props), maxFlowThickness: animationEnabled ? 18 : 12 }, (!adaptiveScalesEnabled) && {
                flowMagnitudeExtent: getFlowMagnitudeExtent(state, props),
            }), { 
                // locationTotalsExtent needs to be always calculated, because locations
                // are not filtered by the viewport (e.g. the connected ones need to be included).
                // Also, the totals cannot be correctly calculated from the flows passed to the layer.
                locationTotalsExtent: getLocationTotalsExtent(state, props), 
                // selectedLocationIds: getExpandedSelection(state, props),
                highlightedLocationId: highlight_1 && highlight_1.type === HighlightType.LOCATION
                    ? highlight_1.locationId
                    : undefined, highlightedFlow: highlight_1 && highlight_1.type === HighlightType.FLOW ? highlight_1.flow : undefined, pickable: true }), (!mapDrawingEnabled && {
                onHover: handleHover,
                onClick: handleClick,
            })), { visible: true, updateTriggers: {
                    onHover: handleHover,
                    onClick: handleClick,
                } })));
        }
        return layers;
    };
    return (React.createElement(NoScrollContainer, { ref: outerRef, onMouseLeave: hideTooltip, className: darkMode ? Classes.DARK : undefined, style: {
            background: darkMode ? Colors.DARK_GRAY1 : Colors.LIGHT_GRAY5,
        } },
        React.createElement(DeckGLOuter, { darkMode: darkMode, baseMapOpacity: state.baseMapOpacity / 100, cursor: mapDrawingEnabled ? 'crosshair' : undefined },
            React.createElement(DeckGL, { ref: deckRef, controller: CONTROLLER_OPTIONS, viewState: viewport, onViewStateChange: handleViewStateChange, layers: getLayers(), ContextProvider: MapContext.Provider, parameters: {
                    clearColor: darkMode ? [0, 0, 0, 1] : [255, 255, 255, 1],
                } },
                mapboxAccessToken && baseMapEnabled && (React.createElement(StaticMap, { mapboxApiAccessToken: mapboxAccessToken, mapStyle: mapboxMapStyle, width: "100%", height: "100%" })),
                mapDrawingEnabled && (React.createElement(MapDrawingEditor, { mapDrawingMode: MapDrawingMode.POLYGON, onFeatureDrawn: handleMapFeatureDrawn })))),
        timeExtent && timeGranularity && totalCountsByTime && selectedTimeRange && (React.createElement(Absolute, { bottom: 20, left: 100, right: 200 },
            React.createElement(Column, { spacing: 10 },
                React.createElement(SelectedTimeRangeBox, { darkMode: darkMode }, selectedTimeRangeToString(selectedTimeRange, timeGranularity)),
                React.createElement(TimelineBox, { darkMode: darkMode },
                    React.createElement(Timeline, { darkMode: darkMode, extent: timeExtent, selectedRange: selectedTimeRange, timeGranularity: timeGranularity, totalCountsByTime: totalCountsByTime, onChange: handleTimeRangeChanged }))))),
        flows && (React.createElement(React.Fragment, null,
            searchBoxLocations && (React.createElement(Absolute, { top: 10, right: 50 },
                React.createElement(BoxStyle, { darkMode: darkMode },
                    React.createElement(LocationsSearchBox, { locationFilterMode: state.locationFilterMode, locations: searchBoxLocations, selectedLocations: state.selectedLocations, onSelectionChanged: handleChangeSelectLocations, onLocationFilterModeChange: handleChangeLocationFilterMode })))),
            React.createElement(Absolute, { top: 10, right: 10 },
                React.createElement(Column, { spacing: 10 },
                    React.createElement(ButtonGroup, { vertical: true },
                        React.createElement(Button, { title: "Zoom in", icon: IconNames.PLUS, onClick: handleZoomIn }),
                        React.createElement(Button, { title: "Zoom out", icon: IconNames.MINUS, onClick: handleZoomOut }),
                        React.createElement(Button, { title: "Reset bearing and pitch", icon: IconNames.COMPASS, onClick: handleResetBearingPitch })),
                    React.createElement(ButtonGroup, { vertical: true },
                        React.createElement(Button, { title: "Filter by drawing a polygon", icon: IconNames.POLYGON_FILTER, active: mapDrawingEnabled, onClick: handleToggleMapDrawing })),
                    !inBrowser && !embed && (React.createElement(ButtonGroup, { vertical: true },
                        React.createElement(SharePopover, null,
                            React.createElement(Button, { title: "Share\u2026", icon: IconNames.SHARE })))))),
            state.locationTotalsEnabled && !embed && (React.createElement(Box, { bottom: 28, right: 0, darkMode: darkMode },
                React.createElement(Collapsible, { darkMode: darkMode, width: 160, direction: Direction.RIGHT },
                    React.createElement(Column, { spacing: 10, padding: 12 },
                        React.createElement(LegendTitle, null, "Location totals"),
                        React.createElement(LocationTotalsLegend, { diff: diffMode, colors: getFlowMapColors(state, props) }))))))),
        !embed && (React.createElement(Absolute, { bottom: 40, left: 10 },
            React.createElement(SettingsPopover, { darkMode: darkMode, state: state, dispatch: dispatch, clusterZoom: getClusterZoom(state, props), availableClusterZoomLevels: getAvailableClusterZoomLevels(state, props), onChangeClusteringAuto: handleChangeClusteringAuto }))),
        showFullscreenButton && (React.createElement(Absolute, { bottom: 30, right: 10 },
            React.createElement(Button, { title: "Open in full-screen mode", onClick: handleFullScreen, icon: IconNames.FULLSCREEN }))),
        spreadSheetKey && !embed && (React.createElement(TitleBox, { top: 60, left: 0, darkMode: darkMode },
            React.createElement(Collapsible, { darkMode: darkMode, width: 300, direction: Direction.LEFT },
                React.createElement(Column, { spacing: 10, padding: "12px 20px" },
                    title && (React.createElement("div", null,
                        React.createElement(Title, null, title),
                        React.createElement(Description, null, description))),
                    flowsSheets && flowsSheets.length > 1 && (React.createElement(HTMLSelect, { value: props.flowsSheet, onChange: handleSelectFlowsSheet, options: flowsSheets.map(function (sheet) { return ({
                            label: sheet,
                            value: sheet,
                        }); }) })),
                    authorUrl ? (React.createElement("div", null, "Created by: ",
                        React.createElement(Away, { href: "" + (authorUrl.indexOf('://') < 0 ? 'http://' : '') + authorUrl }, authorName || 'Author'))) : authorName ? (React.createElement("div", null,
                        "Created by: ",
                        authorName)) : null,
                    sourceName && sourceUrl && (React.createElement("div", null,
                        'Original data source: ',
                        React.createElement(React.Fragment, null,
                            React.createElement(Away, { href: "" + (sourceUrl.indexOf('://') < 0 ? 'http://' : '') + sourceUrl }, sourceName)))),
                    React.createElement("div", null,
                        'Data behind this map is in ',
                        React.createElement(Away, { href: "https://docs.google.com/spreadsheets/d/" + spreadSheetKey }, "this spreadsheet"),
                        ". You can ",
                        React.createElement(Link, { to: "/" }, "publish your own"),
                        " too."),
                    totalFilteredCount != null && totalUnfilteredCount != null && (React.createElement(TotalCount, { darkMode: darkMode }, Math.round(totalFilteredCount) === Math.round(totalUnfilteredCount)
                        ? (_a = config['msg.totalCount.allTrips']) === null || _a === void 0 ? void 0 : _a.replace('{0}', formatCount(totalUnfilteredCount)) : (_b = config['msg.totalCount.countOfTrips']) === null || _b === void 0 ? void 0 : _b.replace('{0}', formatCount(totalFilteredCount)).replace('{1}', formatCount(totalUnfilteredCount)))))))),
        tooltip && React.createElement(Tooltip, __assign({}, tooltip)),
        flowsFetch.loading && React.createElement(LoadingSpinner, null)));
};
function selectedTimeRangeToString(selectedTimeRange, timeGranularity) {
    var interval = timeGranularity.interval, formatFull = timeGranularity.formatFull, order = timeGranularity.order;
    var start = selectedTimeRange[0];
    var end = selectedTimeRange[1];
    if (order >= 3) {
        end = interval.offset(end, -1);
    }
    if (end <= start)
        end = start;
    var startStr = formatFull(start);
    var endStr = formatFull(end);
    if (startStr === endStr)
        return startStr;
    // TODO: split by words, only remove common words
    // let i = 0;
    // while (i < Math.min(startStr.length, endStr.length)) {
    //   if (startStr.charAt(startStr.length - i - 1) !== endStr.charAt(endStr.length - i - 1)) {
    //     break;
    //   }
    //   i++;
    // }
    // if (i > 0) {
    //   return `${startStr.substring(0, startStr.length - i - 1)} - ${endStr}`;
    // }
    return startStr + " \u2013 " + endStr;
}
export default FlowMap;
var templateObject_1;
//# sourceMappingURL=FlowMap.js.map