import { ParametricSelector } from 'reselect';
import { LocationFilterMode, State } from './FlowMap.state';
import { Config, CountByTime, Flow, Location, LocationTotals } from './types';
import * as Cluster from '@flowmap.gl/cluster';
import { ClusterNode } from '@flowmap.gl/cluster';
import { Props } from './FlowMap';
import { TimeGranularity } from './time';
export declare const NUMBER_OF_FLOWS_TO_DISPLAY = 5000;
export declare type Selector<T> = ParametricSelector<State, Props, T>;
export declare const getFetchedFlows: (state: State, props: Props) => Flow[] | undefined;
export declare const getFetchedLocations: (state: State, props: Props) => Location[] | undefined;
export declare const getSelectedLocations: (state: State, props: Props) => string[] | undefined;
export declare const getLocationFilterMode: (state: State, props: Props) => LocationFilterMode;
export declare const getClusteringEnabled: (state: State, props: Props) => boolean;
export declare const getLocationTotalsEnabled: (state: State, props: Props) => boolean;
export declare const getZoom: (state: State, props: Props) => number;
export declare const getConfig: (state: State, props: Props) => Record<string, string | undefined>;
export declare const getViewport: (state: State, props: Props) => import("./types").ViewportProps;
export declare const getSelectedTimeRange: (state: State, props: Props) => [Date, Date] | undefined;
export declare const getInvalidLocationIds: Selector<string[] | undefined>;
export declare const getLocations: Selector<Location[] | undefined>;
export declare const getLocationIds: Selector<Set<string> | undefined>;
export declare const getSelectedLocationsSet: Selector<Set<string> | undefined>;
export declare const getSortedFlowsForKnownLocations: Selector<Flow[] | undefined>;
export declare const getTimeGranularity: Selector<TimeGranularity | undefined>;
export declare const getTimeExtent: Selector<[Date, Date] | undefined>;
export declare const getSortedFlowsForKnownLocationsFilteredByTime: Selector<Flow[] | undefined>;
export declare const getLocationsHavingFlows: Selector<Location[] | undefined>;
export declare const getClusterIndex: Selector<Cluster.ClusterIndex | undefined>;
export declare const getAvailableClusterZoomLevels: import("reselect").OutputParametricSelector<State, Props, number[] | undefined, (res1: number, res2: Cluster.ClusterIndex | undefined, res3: string[] | undefined) => number[] | undefined>;
export declare function getClusterZoom(state: State, props: Props): number | undefined;
export declare const getLocationsForSearchBox: Selector<(Location | Cluster.Cluster)[] | undefined>;
export declare const getDiffMode: Selector<boolean>;
export declare const getColorSchemeKey: Selector<string | undefined>;
export declare const getDarkMode: Selector<boolean>;
export declare const getFadeEnabled: Selector<boolean>;
export declare const getFadeAmount: Selector<number>;
export declare const getAnimate: Selector<boolean>;
export declare const getFlowMapColors: import("reselect").OutputParametricSelector<State, Props, import("@flowmap.gl/core").Colors | import("@flowmap.gl/core").DiffColors, (res1: Record<string, string | undefined>, res2: boolean, res3: string | undefined, res4: boolean, res5: boolean, res6: number, res7: boolean) => import("@flowmap.gl/core").Colors | import("@flowmap.gl/core").DiffColors>;
export declare const getMapboxMapStyle: import("reselect").OutputParametricSelector<State, Props, string, (res1: Record<string, string | undefined>, res2: boolean) => string>;
export declare const getUnknownLocations: Selector<Set<string> | undefined>;
export declare const getSortedAggregatedFilteredFlows: Selector<Flow[] | undefined>;
export declare const getFlowMagnitudeExtent: Selector<[number, number] | undefined>;
export declare const getExpandedSelectedLocationsSet: Selector<Set<string> | undefined>;
export declare const getTotalCountsByTime: Selector<CountByTime[] | undefined>;
export declare const getMaxLocationCircleSize: Selector<number>;
export declare const getLocationTotals: Selector<Map<string, LocationTotals> | undefined>;
declare type KDBushTree = any;
export declare const getLocationsTree: Selector<KDBushTree>;
export declare function getLocationsInBbox(tree: KDBushTree, bbox: [number, number, number, number]): Location[] | undefined;
export declare const getLocationsForFlowMapLayer: Selector<Location[] | ClusterNode[] | undefined>;
export declare const getFlowsSheets: (config: Config) => string[] | undefined;
export declare const getTotalUnfilteredCount: Selector<number | undefined>;
export declare const getTotalFilteredCount: Selector<number | undefined>;
export declare function getLocationTotalsExtent(state: State, props: Props): [number, number] | undefined;
export declare const getFlowsForFlowMapLayer: Selector<Flow[] | undefined>;
export {};
//# sourceMappingURL=FlowMap.selectors.d.ts.map