var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
import { createSelector, createSelectorCreator, defaultMemoize, } from 'reselect';
import { LocationFilterMode, MAX_ZOOM_LEVEL } from './FlowMap.state';
import { ConfigPropName, getFlowDestId, getFlowMagnitude, getFlowOriginId, getFlowTime, getLocationCentroid, getLocationId, isLocationCluster, } from './types';
import * as Cluster from '@flowmap.gl/cluster';
import { findAppropriateZoomLevel, isCluster } from '@flowmap.gl/cluster';
import getColors from './colors';
import { DEFAULT_MAP_STYLE_DARK, DEFAULT_MAP_STYLE_LIGHT, parseBoolConfigProp } from './config';
import { nest } from 'd3-collection';
import { bounds } from '@mapbox/geo-viewport';
import KDBush from 'kdbush';
import { descending, min } from 'd3-array';
import { csvParseRows } from 'd3-dsv';
import { getTimeGranularityByOrder, getTimeGranularityForDate } from './time';
export var NUMBER_OF_FLOWS_TO_DISPLAY = 5000;
export var getFetchedFlows = function (state, props) { return props.flowsFetch.value; };
export var getFetchedLocations = function (state, props) { return props.locationsFetch.value; };
export var getSelectedLocations = function (state, props) { return state.selectedLocations; };
export var getLocationFilterMode = function (state, props) { return state.locationFilterMode; };
export var getClusteringEnabled = function (state, props) { return state.clusteringEnabled; };
export var getLocationTotalsEnabled = function (state, props) { return state.locationTotalsEnabled; };
export var getZoom = function (state, props) { return state.viewport.zoom; };
export var getConfig = function (state, props) { return props.config; };
export var getViewport = function (state, props) { return state.viewport; };
export var getSelectedTimeRange = function (state, props) { return state.selectedTimeRange; };
export var getInvalidLocationIds = createSelector(getFetchedLocations, function (locations) {
    var e_1, _a;
    if (!locations)
        return undefined;
    var invalid = [];
    try {
        for (var locations_1 = __values(locations), locations_1_1 = locations_1.next(); !locations_1_1.done; locations_1_1 = locations_1.next()) {
            var location_1 = locations_1_1.value;
            if (!(-90 <= location_1.lat && location_1.lat <= 90) ||
                !(-180 <= location_1.lon && location_1.lon <= 180)) {
                invalid.push(location_1.id);
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (locations_1_1 && !locations_1_1.done && (_a = locations_1.return)) _a.call(locations_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return invalid.length > 0 ? invalid : undefined;
});
export var getLocations = createSelector(getFetchedLocations, getInvalidLocationIds, function (locations, invalidIds) {
    if (!locations)
        return undefined;
    if (!invalidIds || invalidIds.length === 0)
        return locations;
    var invalid = new Set(invalidIds);
    return locations.filter(function (location) { return !invalid.has(getLocationId(location)); });
});
export var getLocationIds = createSelector(getLocations, function (locations) { return (locations ? new Set(locations.map(getLocationId)) : undefined); });
export var getSelectedLocationsSet = createSelector(getSelectedLocations, function (ids) {
    return ids && ids.length > 0 ? new Set(ids) : undefined;
});
export var getSortedFlowsForKnownLocations = createSelector(getFetchedFlows, getLocationIds, function (flows, ids) {
    if (!ids || !flows)
        return undefined;
    return flows
        .filter(function (flow) { return ids.has(getFlowOriginId(flow)) && ids.has(getFlowDestId(flow)); })
        .sort(function (a, b) { return descending(Math.abs(getFlowMagnitude(a)), Math.abs(getFlowMagnitude(b))); });
});
var getActualTimeExtent = createSelector(getSortedFlowsForKnownLocations, function (flows) {
    var e_2, _a;
    if (!flows)
        return undefined;
    var start = null;
    var end = null;
    try {
        for (var flows_1 = __values(flows), flows_1_1 = flows_1.next(); !flows_1_1.done; flows_1_1 = flows_1.next()) {
            var time = flows_1_1.value.time;
            if (time) {
                if (start == null || start > time)
                    start = time;
                if (end == null || end < time)
                    end = time;
            }
        }
    }
    catch (e_2_1) { e_2 = { error: e_2_1 }; }
    finally {
        try {
            if (flows_1_1 && !flows_1_1.done && (_a = flows_1.return)) _a.call(flows_1);
        }
        finally { if (e_2) throw e_2.error; }
    }
    if (!start || !end)
        return undefined;
    return [start, end];
});
export var getTimeGranularity = createSelector(getSortedFlowsForKnownLocations, getActualTimeExtent, function (flows, timeExtent) {
    if (!flows || !timeExtent)
        return undefined;
    var minOrder = min(flows, function (d) { return getTimeGranularityForDate(getFlowTime(d)).order; });
    if (minOrder == null)
        return undefined;
    return getTimeGranularityByOrder(minOrder);
});
export var getTimeExtent = createSelector(getActualTimeExtent, getTimeGranularity, function (timeExtent, timeGranularity) {
    if (!timeExtent || !(timeGranularity === null || timeGranularity === void 0 ? void 0 : timeGranularity.interval))
        return undefined;
    var interval = timeGranularity.interval;
    return [timeExtent[0], interval.offset(interval.floor(timeExtent[1]), 1)];
});
export var getSortedFlowsForKnownLocationsFilteredByTime = createSelector(getSortedFlowsForKnownLocations, getTimeExtent, getSelectedTimeRange, function (flows, timeExtent, timeRange) {
    if (!flows)
        return undefined;
    if (!timeExtent ||
        !timeRange ||
        (timeExtent[0] === timeRange[0] && timeExtent[1] === timeRange[1])) {
        return flows;
    }
    return flows.filter(function (flow) {
        var time = getFlowTime(flow);
        return time && timeRange[0] <= time && time < timeRange[1];
    });
});
export var getLocationsHavingFlows = createSelector(getSortedFlowsForKnownLocations, getLocations, function (flows, locations) {
    var e_3, _a;
    if (!locations || !flows)
        return locations;
    var withFlows = new Set();
    try {
        for (var flows_2 = __values(flows), flows_2_1 = flows_2.next(); !flows_2_1.done; flows_2_1 = flows_2.next()) {
            var flow = flows_2_1.value;
            withFlows.add(getFlowOriginId(flow));
            withFlows.add(getFlowDestId(flow));
        }
    }
    catch (e_3_1) { e_3 = { error: e_3_1 }; }
    finally {
        try {
            if (flows_2_1 && !flows_2_1.done && (_a = flows_2.return)) _a.call(flows_2);
        }
        finally { if (e_3) throw e_3.error; }
    }
    return locations.filter(function (location) { return withFlows.has(getLocationId(location)); });
});
export var getClusterIndex = createSelector(getLocationsHavingFlows, getSortedFlowsForKnownLocations, function (locations, flows) {
    var e_4, _a, e_5, _b;
    if (!locations || !flows)
        return undefined;
    var getLocationWeight = Cluster.makeLocationWeightGetter(flows, {
        getFlowOriginId: getFlowOriginId,
        getFlowDestId: getFlowDestId,
        getFlowMagnitude: getFlowMagnitude,
    });
    var clusterLevels = Cluster.clusterLocations(locations, { getLocationId: getLocationId, getLocationCentroid: getLocationCentroid }, getLocationWeight, {
        maxZoom: MAX_ZOOM_LEVEL,
    });
    var clusterIndex = Cluster.buildIndex(clusterLevels);
    var locationsById = nest()
        .key(function (d) { return d.id; })
        .rollup(function (_a) {
        var _b = __read(_a, 1), d = _b[0];
        return d;
    })
        .object(locations);
    // Adding meaningful names
    var getName = function (id) {
        var loc = locationsById[id];
        if (loc)
            return loc.name || loc.id || id;
        return "#" + id;
    };
    try {
        for (var clusterLevels_1 = __values(clusterLevels), clusterLevels_1_1 = clusterLevels_1.next(); !clusterLevels_1_1.done; clusterLevels_1_1 = clusterLevels_1.next()) {
            var level = clusterLevels_1_1.value;
            var _loop_1 = function (node) {
                // Here mutating the nodes (adding names)
                if (isCluster(node)) {
                    var leaves = clusterIndex.expandCluster(node);
                    var topId_1 = leaves.reduce(function (m, d) {
                        return !m || getLocationWeight(d) > getLocationWeight(m) ? d : m;
                    });
                    var otherId = leaves.length === 2 && leaves.find(function (id) { return id !== topId_1; });
                    node.name = "\"" + getName(topId_1) + "\" and " + (otherId ? "\"" + getName(otherId) + "\"" : leaves.length - 1 + " others");
                }
                else {
                    node.name = getName(node.id);
                }
            };
            try {
                for (var _c = (e_5 = void 0, __values(level.nodes)), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var node = _d.value;
                    _loop_1(node);
                }
            }
            catch (e_5_1) { e_5 = { error: e_5_1 }; }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return)) _b.call(_c);
                }
                finally { if (e_5) throw e_5.error; }
            }
        }
    }
    catch (e_4_1) { e_4 = { error: e_4_1 }; }
    finally {
        try {
            if (clusterLevels_1_1 && !clusterLevels_1_1.done && (_a = clusterLevels_1.return)) _a.call(clusterLevels_1);
        }
        finally { if (e_4) throw e_4.error; }
    }
    return clusterIndex;
});
export var getAvailableClusterZoomLevels = createSelector(getZoom, getClusterIndex, getSelectedLocations, function (mapZoom, clusterIndex, selectedLocations) {
    var e_6, _a;
    if (!clusterIndex) {
        return undefined;
    }
    var maxZoom = Number.POSITIVE_INFINITY;
    var minZoom = Number.NEGATIVE_INFINITY;
    var adjust = function (zoneId) {
        var cluster = clusterIndex.getClusterById(zoneId);
        if (cluster) {
            minZoom = Math.max(minZoom, cluster.zoom);
            maxZoom = Math.min(maxZoom, cluster.zoom);
        }
        else {
            var zoom = clusterIndex.getMinZoomForLocation(zoneId);
            minZoom = Math.max(minZoom, zoom);
        }
    };
    if (selectedLocations) {
        try {
            for (var selectedLocations_1 = __values(selectedLocations), selectedLocations_1_1 = selectedLocations_1.next(); !selectedLocations_1_1.done; selectedLocations_1_1 = selectedLocations_1.next()) {
                var id = selectedLocations_1_1.value;
                adjust(id);
            }
        }
        catch (e_6_1) { e_6 = { error: e_6_1 }; }
        finally {
            try {
                if (selectedLocations_1_1 && !selectedLocations_1_1.done && (_a = selectedLocations_1.return)) _a.call(selectedLocations_1);
            }
            finally { if (e_6) throw e_6.error; }
        }
    }
    return clusterIndex.availableZoomLevels.filter(function (level) { return minZoom <= level && level <= maxZoom; });
});
var _getClusterZoom = createSelector(getClusterIndex, getZoom, getAvailableClusterZoomLevels, function (clusterIndex, mapZoom, availableClusterZoomLevels) {
    if (!clusterIndex)
        return undefined;
    if (!availableClusterZoomLevels) {
        return undefined;
    }
    var clusterZoom = findAppropriateZoomLevel(availableClusterZoomLevels, mapZoom);
    return clusterZoom;
});
export function getClusterZoom(state, props) {
    if (!state.clusteringEnabled)
        return undefined;
    if (state.clusteringAuto || state.manualClusterZoom == null) {
        return _getClusterZoom(state, props);
    }
    return state.manualClusterZoom;
}
export var getLocationsForSearchBox = createSelector(getClusteringEnabled, getLocationsHavingFlows, getSelectedLocations, getClusterZoom, getClusterIndex, function (clusteringEnabled, locations, selectedLocations, clusterZoom, clusterIndex) {
    var e_7, _a;
    if (!locations)
        return undefined;
    var result = locations;
    if (clusteringEnabled) {
        if (clusterIndex) {
            var zoomItems = clusterIndex.getClusterNodesFor(clusterZoom);
            if (zoomItems) {
                result = result.concat(zoomItems.filter(Cluster.isCluster));
            }
        }
    }
    if (result && clusterIndex && selectedLocations) {
        var toAppend = [];
        var _loop_2 = function (id) {
            var cluster = clusterIndex.getClusterById(id);
            if (cluster && !result.find(function (d) { return d.id === id; })) {
                toAppend.push(cluster);
            }
        };
        try {
            for (var selectedLocations_2 = __values(selectedLocations), selectedLocations_2_1 = selectedLocations_2.next(); !selectedLocations_2_1.done; selectedLocations_2_1 = selectedLocations_2.next()) {
                var id = selectedLocations_2_1.value;
                _loop_2(id);
            }
        }
        catch (e_7_1) { e_7 = { error: e_7_1 }; }
        finally {
            try {
                if (selectedLocations_2_1 && !selectedLocations_2_1.done && (_a = selectedLocations_2.return)) _a.call(selectedLocations_2);
            }
            finally { if (e_7) throw e_7.error; }
        }
        if (toAppend.length > 0) {
            result = result.concat(toAppend);
        }
    }
    return result;
});
export var getDiffMode = createSelector(getFetchedFlows, function (flows) {
    if (flows && flows.find(function (f) { return getFlowMagnitude(f) < 0; })) {
        return true;
    }
    return false;
});
export var getColorSchemeKey = function (state, props) {
    return state.colorSchemeKey;
};
export var getDarkMode = function (state, props) { return state.darkMode; };
export var getFadeEnabled = function (state, props) { return state.fadeEnabled; };
export var getFadeAmount = function (state, props) { return state.fadeAmount; };
export var getAnimate = function (state, props) { return state.animationEnabled; };
export var getFlowMapColors = createSelector(getConfig, getDiffMode, getColorSchemeKey, getDarkMode, getFadeEnabled, getFadeAmount, getAnimate, getColors);
export var getMapboxMapStyle = createSelector(getConfig, getDarkMode, function (config, darkMode) {
    var configMapStyle = config[ConfigPropName.MAPBOX_MAP_STYLE];
    if (configMapStyle) {
        if (darkMode === parseBoolConfigProp(config[ConfigPropName.COLORS_DARK_MODE])) {
            return configMapStyle;
        }
    }
    return darkMode ? DEFAULT_MAP_STYLE_DARK : DEFAULT_MAP_STYLE_LIGHT;
});
export var getUnknownLocations = createSelector(getLocationIds, getFetchedFlows, getSortedFlowsForKnownLocations, function (ids, flows, flowsForKnownLocations) {
    var e_8, _a;
    if (!ids || !flows)
        return undefined;
    if (flowsForKnownLocations && flows.length === flowsForKnownLocations.length)
        return undefined;
    var missing = new Set();
    try {
        for (var flows_3 = __values(flows), flows_3_1 = flows_3.next(); !flows_3_1.done; flows_3_1 = flows_3.next()) {
            var flow = flows_3_1.value;
            if (!ids.has(getFlowOriginId(flow)))
                missing.add(getFlowOriginId(flow));
            if (!ids.has(getFlowDestId(flow)))
                missing.add(getFlowDestId(flow));
        }
    }
    catch (e_8_1) { e_8 = { error: e_8_1 }; }
    finally {
        try {
            if (flows_3_1 && !flows_3_1.done && (_a = flows_3.return)) _a.call(flows_3);
        }
        finally { if (e_8) throw e_8.error; }
    }
    return missing;
});
function aggregateFlows(flows) {
    var e_9, _a, e_10, _b;
    // Sum up flows with same origin, dest
    var byOriginDest = nest()
        .key(getFlowOriginId)
        .key(getFlowDestId)
        .rollup(function (ff) {
        var origin = getFlowOriginId(ff[0]);
        var dest = getFlowDestId(ff[0]);
        return {
            origin: origin,
            dest: dest,
            count: ff.reduce(function (m, f) {
                var count = getFlowMagnitude(f);
                if (count) {
                    if (!isNaN(count) && isFinite(count))
                        return m + count;
                }
                return m;
            }, 0),
            time: undefined,
        };
    })
        .entries(flows);
    var rv = [];
    try {
        for (var byOriginDest_1 = __values(byOriginDest), byOriginDest_1_1 = byOriginDest_1.next(); !byOriginDest_1_1.done; byOriginDest_1_1 = byOriginDest_1.next()) {
            var values = byOriginDest_1_1.value.values;
            try {
                for (var values_1 = (e_10 = void 0, __values(values)), values_1_1 = values_1.next(); !values_1_1.done; values_1_1 = values_1.next()) {
                    var value = values_1_1.value.value;
                    rv.push(value);
                }
            }
            catch (e_10_1) { e_10 = { error: e_10_1 }; }
            finally {
                try {
                    if (values_1_1 && !values_1_1.done && (_b = values_1.return)) _b.call(values_1);
                }
                finally { if (e_10) throw e_10.error; }
            }
        }
    }
    catch (e_9_1) { e_9 = { error: e_9_1 }; }
    finally {
        try {
            if (byOriginDest_1_1 && !byOriginDest_1_1.done && (_a = byOriginDest_1.return)) _a.call(byOriginDest_1);
        }
        finally { if (e_9) throw e_9.error; }
    }
    return rv;
}
export var getSortedAggregatedFilteredFlows = createSelector(getClusterIndex, getClusteringEnabled, getSortedFlowsForKnownLocationsFilteredByTime, getClusterZoom, getTimeExtent, function (clusterTree, isClusteringEnabled, flows, clusterZoom, timeExtent) {
    if (!flows)
        return undefined;
    var aggregated;
    if (isClusteringEnabled && clusterTree && clusterZoom != null) {
        aggregated = clusterTree.aggregateFlows(timeExtent != null
            ? aggregateFlows(flows) // clusterTree.aggregateFlows won't aggregate unclustered across time
            : flows, clusterZoom, {
            getFlowOriginId: getFlowOriginId,
            getFlowDestId: getFlowDestId,
            getFlowMagnitude: getFlowMagnitude,
        });
    }
    else {
        aggregated = aggregateFlows(flows);
    }
    aggregated.sort(function (a, b) {
        return descending(Math.abs(getFlowMagnitude(a)), Math.abs(getFlowMagnitude(b)));
    });
    return aggregated;
});
export var getFlowMagnitudeExtent = createSelector(getSortedAggregatedFilteredFlows, getSelectedLocationsSet, getLocationFilterMode, function (flows, selectedLocationsSet, locationFilterMode) {
    var e_11, _a;
    if (!flows)
        return undefined;
    var rv = undefined;
    try {
        for (var flows_4 = __values(flows), flows_4_1 = flows_4.next(); !flows_4_1.done; flows_4_1 = flows_4.next()) {
            var f = flows_4_1.value;
            if (getFlowOriginId(f) !== getFlowDestId(f)
                && isFlowInSelection(f, selectedLocationsSet, locationFilterMode)) {
                var count = getFlowMagnitude(f);
                if (rv == null) {
                    rv = [count, count];
                }
                else {
                    if (count < rv[0])
                        rv[0] = count;
                    if (count > rv[1])
                        rv[1] = count;
                }
            }
        }
    }
    catch (e_11_1) { e_11 = { error: e_11_1 }; }
    finally {
        try {
            if (flows_4_1 && !flows_4_1.done && (_a = flows_4.return)) _a.call(flows_4);
        }
        finally { if (e_11) throw e_11.error; }
    }
    return rv;
});
export var getExpandedSelectedLocationsSet = createSelector(getClusteringEnabled, getSelectedLocationsSet, getClusterIndex, function (clusteringEnabled, selectedLocations, clusterIndex) {
    var e_12, _a, e_13, _b;
    if (!selectedLocations || !clusterIndex) {
        return selectedLocations;
    }
    var result = new Set();
    try {
        for (var selectedLocations_3 = __values(selectedLocations), selectedLocations_3_1 = selectedLocations_3.next(); !selectedLocations_3_1.done; selectedLocations_3_1 = selectedLocations_3.next()) {
            var locationId = selectedLocations_3_1.value;
            var cluster = clusterIndex.getClusterById(locationId);
            if (cluster) {
                var expanded = clusterIndex.expandCluster(cluster);
                try {
                    for (var expanded_1 = (e_13 = void 0, __values(expanded)), expanded_1_1 = expanded_1.next(); !expanded_1_1.done; expanded_1_1 = expanded_1.next()) {
                        var id = expanded_1_1.value;
                        result.add(id);
                    }
                }
                catch (e_13_1) { e_13 = { error: e_13_1 }; }
                finally {
                    try {
                        if (expanded_1_1 && !expanded_1_1.done && (_b = expanded_1.return)) _b.call(expanded_1);
                    }
                    finally { if (e_13) throw e_13.error; }
                }
            }
            else {
                result.add(locationId);
            }
        }
    }
    catch (e_12_1) { e_12 = { error: e_12_1 }; }
    finally {
        try {
            if (selectedLocations_3_1 && !selectedLocations_3_1.done && (_a = selectedLocations_3.return)) _a.call(selectedLocations_3);
        }
        finally { if (e_12) throw e_12.error; }
    }
    return result;
});
export var getTotalCountsByTime = createSelector(getSortedFlowsForKnownLocations, getTimeGranularity, getTimeExtent, getExpandedSelectedLocationsSet, getLocationFilterMode, function (flows, timeGranularity, timeExtent, selectedLocationSet, locationFilterMode) {
    if (!flows || !timeGranularity || !timeExtent)
        return undefined;
    var byTime = flows.reduce(function (m, flow) {
        var _a;
        if (isFlowInSelection(flow, selectedLocationSet, locationFilterMode)) {
            var key = timeGranularity.interval(getFlowTime(flow)).getTime();
            m.set(key, ((_a = m.get(key)) !== null && _a !== void 0 ? _a : 0) + getFlowMagnitude(flow));
        }
        return m;
    }, new Map());
    return Array.from(byTime.entries()).map(function (_a) {
        var _b = __read(_a, 2), millis = _b[0], count = _b[1];
        return ({
            time: new Date(millis),
            count: count,
        });
    });
});
export var getMaxLocationCircleSize = createSelector(getLocationTotalsEnabled, function (locationTotalsEnabled) { return (locationTotalsEnabled ? 15 : 0); });
var getViewportBoundingBox = createSelector(getViewport, getMaxLocationCircleSize, function (viewport, maxLocationCircleSize) {
    var pad = maxLocationCircleSize;
    return bounds([viewport.longitude, viewport.latitude], viewport.zoom, [viewport.width + pad * 2, viewport.height + pad * 2], 512);
});
var getLocationsForZoom = createSelector(getClusteringEnabled, getLocationsHavingFlows, getClusterIndex, getClusterZoom, function (clusteringEnabled, locationsHavingFlows, clusterIndex, clusterZoom) {
    if (clusteringEnabled && clusterIndex) {
        return clusterIndex.getClusterNodesFor(clusterZoom);
    }
    else {
        return locationsHavingFlows;
    }
});
export var getLocationTotals = createSelector(getLocationsForZoom, getSortedAggregatedFilteredFlows, getSelectedLocationsSet, getLocationFilterMode, function (locations, flows, selectedLocationsSet, locationFilterMode) {
    var e_14, _a;
    if (!flows)
        return undefined;
    var totals = new Map();
    var add = function (id, d) {
        var _a;
        var rv = (_a = totals.get(id)) !== null && _a !== void 0 ? _a : { incoming: 0, outgoing: 0, within: 0 };
        if (d.incoming != null)
            rv.incoming += d.incoming;
        if (d.outgoing != null)
            rv.outgoing += d.outgoing;
        if (d.within != null)
            rv.within += d.within;
        return rv;
    };
    try {
        for (var flows_5 = __values(flows), flows_5_1 = flows_5.next(); !flows_5_1.done; flows_5_1 = flows_5.next()) {
            var f = flows_5_1.value;
            if (isFlowInSelection(f, selectedLocationsSet, locationFilterMode)) {
                var originId = getFlowOriginId(f);
                var destId = getFlowDestId(f);
                var count = getFlowMagnitude(f);
                if (originId === destId) {
                    totals.set(originId, add(originId, { within: count }));
                }
                else {
                    totals.set(originId, add(originId, { outgoing: count }));
                    totals.set(destId, add(destId, { incoming: count }));
                }
            }
        }
    }
    catch (e_14_1) { e_14 = { error: e_14_1 }; }
    finally {
        try {
            if (flows_5_1 && !flows_5_1.done && (_a = flows_5.return)) _a.call(flows_5);
        }
        finally { if (e_14) throw e_14.error; }
    }
    return totals;
});
export var getLocationsTree = createSelector(getLocationsForZoom, function (locations) {
    if (!locations) {
        return undefined;
    }
    return new KDBush(locations, function (location) {
        return lngX(isLocationCluster(location) ? location.centroid[0] : location.lon);
    }, function (location) {
        return latY(isLocationCluster(location) ? location.centroid[1] : location.lat);
    });
});
function _getLocationsInBboxIndices(tree, bbox) {
    if (!tree)
        return undefined;
    var _a = __read(bbox, 4), lon1 = _a[0], lat1 = _a[1], lon2 = _a[2], lat2 = _a[3];
    var _b = __read([lngX(lon1), latY(lat1), lngX(lon2), latY(lat2)], 4), x1 = _b[0], y1 = _b[1], x2 = _b[2], y2 = _b[3];
    return tree.range(Math.min(x1, x2), Math.min(y1, y2), Math.max(x1, x2), Math.max(y1, y2));
}
export function getLocationsInBbox(tree, bbox) {
    if (!tree)
        return undefined;
    return _getLocationsInBboxIndices(tree, bbox).map(function (idx) { return tree.points[idx]; });
}
var _getLocationIdsInViewport = createSelector(getLocationsTree, getViewportBoundingBox, function (tree, bbox) {
    var ids = _getLocationsInBboxIndices(tree, bbox);
    if (ids) {
        return new Set(ids.map(function (idx) { return tree.points[idx].id; }));
    }
    return undefined;
});
var getLocationIdsInViewport = createSelectorCreator(
// @ts-ignore
defaultMemoize, function (s1, s2, index) {
    var e_15, _a;
    if (s1 === s2)
        return true;
    if (s1 == null || s2 == null)
        return false;
    if (s1.size !== s2.size)
        return false;
    try {
        for (var s1_1 = __values(s1), s1_1_1 = s1_1.next(); !s1_1_1.done; s1_1_1 = s1_1.next()) {
            var item = s1_1_1.value;
            if (!s2.has(item))
                return false;
        }
    }
    catch (e_15_1) { e_15 = { error: e_15_1 }; }
    finally {
        try {
            if (s1_1_1 && !s1_1_1.done && (_a = s1_1.return)) _a.call(s1_1);
        }
        finally { if (e_15) throw e_15.error; }
    }
    return true;
})(_getLocationIdsInViewport, function (locationIds) {
    if (!locationIds)
        return undefined;
    return locationIds;
});
export var getLocationsForFlowMapLayer = createSelector(getLocationsForZoom, getLocationIdsInViewport, function (locations, locationIdsInViewport) {
    if (!locations)
        return undefined;
    if (!locationIdsInViewport)
        return locations;
    if (locationIdsInViewport.size === locations.length)
        return locations;
    // const filtered = [];
    // for (const loc of locations) {
    //   if (locationIdsInViewport.has(loc.id)) {
    //     filtered.push(loc);
    //   }
    // }
    // return filtered;
    // @ts-ignore
    // return locations.filter(
    //   (loc: Location | ClusterNode) => locationIdsInViewport.has(loc.id)
    // );
    // TODO: return location in viewport + "connected" ones
    return locations;
});
export var getFlowsSheets = defaultMemoize(function (config) {
    var sheets = config[ConfigPropName.FLOWS_SHEETS];
    if (sheets) {
        return csvParseRows(sheets)[0].map(function (s) { return s.trim(); });
    }
    return undefined;
});
function isFlowInSelection(flow, selectedLocationsSet, locationFilterMode) {
    var origin = flow.origin, dest = flow.dest;
    if (selectedLocationsSet) {
        switch (locationFilterMode) {
            case LocationFilterMode.ALL:
                return selectedLocationsSet.has(origin) || selectedLocationsSet.has(dest);
            case LocationFilterMode.BETWEEN:
                return selectedLocationsSet.has(origin) && selectedLocationsSet.has(dest);
            case LocationFilterMode.INCOMING:
                return selectedLocationsSet.has(dest);
            case LocationFilterMode.OUTGOING:
                return selectedLocationsSet.has(origin);
        }
    }
    return true;
}
export var getTotalUnfilteredCount = createSelector(getSortedFlowsForKnownLocations, function (flows) {
    if (!flows)
        return undefined;
    return flows.reduce(function (m, flow) { return m + getFlowMagnitude(flow); }, 0);
});
export var getTotalFilteredCount = createSelector(getSortedAggregatedFilteredFlows, getSelectedLocationsSet, getLocationFilterMode, function (flows, selectedLocationSet, locationFilterMode) {
    if (!flows)
        return undefined;
    var count = flows.reduce(function (m, flow) {
        if (isFlowInSelection(flow, selectedLocationSet, locationFilterMode)) {
            return m + getFlowMagnitude(flow);
        }
        return m;
    }, 0);
    return count;
});
function calcLocationTotalsExtent(locationTotals, locationIdsInViewport) {
    var e_16, _a;
    if (!locationTotals)
        return undefined;
    var rv = undefined;
    try {
        for (var _b = __values(locationTotals.entries()), _c = _b.next(); !_c.done; _c = _b.next()) {
            var _d = __read(_c.value, 2), id = _d[0], _e = _d[1], incoming = _e.incoming, outgoing = _e.outgoing, within = _e.within;
            if (locationIdsInViewport == null || locationIdsInViewport.has(id)) {
                var lo = Math.min(incoming + within, outgoing + within, within);
                var hi = Math.max(incoming + within, outgoing + within, within);
                if (!rv) {
                    rv = [lo, hi];
                }
                else {
                    if (lo < rv[0])
                        rv[0] = lo;
                    if (hi > rv[1])
                        rv[1] = hi;
                }
            }
        }
    }
    catch (e_16_1) { e_16 = { error: e_16_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_16) throw e_16.error; }
    }
    return rv;
}
var _getLocationTotalsExtent = createSelector(getLocationTotals, function (locationTotals) {
    return calcLocationTotalsExtent(locationTotals, undefined);
});
var _getLocationTotalsForViewportExtent = createSelector(getLocationTotals, getLocationIdsInViewport, function (locationTotals, locationsInViewport) {
    return calcLocationTotalsExtent(locationTotals, locationsInViewport);
});
export function getLocationTotalsExtent(state, props) {
    if (state.adaptiveScalesEnabled) {
        return _getLocationTotalsForViewportExtent(state, props);
    }
    else {
        return _getLocationTotalsExtent(state, props);
    }
}
export var getFlowsForFlowMapLayer = createSelector(getSortedAggregatedFilteredFlows, getLocationIdsInViewport, getSelectedLocationsSet, getLocationFilterMode, function (flows, locationIdsInViewport, selectedLocationsSet, locationFilterMode) {
    var e_17, _a;
    if (!flows || !locationIdsInViewport)
        return undefined;
    var picked = [];
    var pickedCount = 0;
    try {
        for (var flows_6 = __values(flows), flows_6_1 = flows_6.next(); !flows_6_1.done; flows_6_1 = flows_6.next()) {
            var flow = flows_6_1.value;
            var origin_1 = flow.origin, dest = flow.dest;
            if (locationIdsInViewport.has(origin_1) || locationIdsInViewport.has(dest)) {
                var pick = isFlowInSelection(flow, selectedLocationsSet, locationFilterMode);
                if (pick) {
                    picked.push(flow);
                    if (origin_1 !== dest) {
                        // exclude self-loops from count
                        pickedCount++;
                    }
                }
            }
            // Only keep top
            if (pickedCount > NUMBER_OF_FLOWS_TO_DISPLAY)
                break;
        }
    }
    catch (e_17_1) { e_17 = { error: e_17_1 }; }
    finally {
        try {
            if (flows_6_1 && !flows_6_1.done && (_a = flows_6.return)) _a.call(flows_6);
        }
        finally { if (e_17) throw e_17.error; }
    }
    return picked;
});
// longitude/latitude to spherical mercator in [0..1] range
function lngX(lng) {
    return lng / 360 + 0.5;
}
function latY(lat) {
    var sin = Math.sin((lat * Math.PI) / 180);
    var y = 0.5 - (0.25 * Math.log((1 + sin) / (1 - sin))) / Math.PI;
    return y < 0 ? 0 : y > 1 ? 1 : y;
}
//# sourceMappingURL=FlowMap.selectors.js.map