var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
import { FlyToInterpolator } from 'react-map-gl';
import { ConfigPropName } from './types';
import * as queryString from 'query-string';
import { viewport } from '@mapbox/geo-viewport';
import { parseBoolConfigProp, parseNumberConfigProp } from './config';
import { COLOR_SCHEME_KEYS } from './colors';
import { csvFormatRows, csvParseRows } from 'd3-dsv';
import { easeCubic } from 'd3-ease';
import { timeFormat, timeParse } from 'd3-time-format';
export var MIN_ZOOM_LEVEL = 0;
export var MAX_ZOOM_LEVEL = 20;
export var MIN_PITCH = 0;
export var MAX_PITCH = +60;
var TIME_QUERY_FORMAT = '%Y%m%dT%H%M%S';
var timeToQuery = timeFormat(TIME_QUERY_FORMAT);
var timeFromQuery = timeParse(TIME_QUERY_FORMAT);
export function mapTransition(duration) {
    if (duration === void 0) { duration = 500; }
    return {
        transitionDuration: 500,
        transitionInterpolator: new FlyToInterpolator(),
        transitionEasing: easeCubic,
    };
}
export var HighlightType;
(function (HighlightType) {
    HighlightType["LOCATION"] = "location";
    HighlightType["FLOW"] = "flow";
})(HighlightType || (HighlightType = {}));
export var LocationFilterMode;
(function (LocationFilterMode) {
    LocationFilterMode["ALL"] = "ALL";
    LocationFilterMode["INCOMING"] = "INCOMING";
    LocationFilterMode["OUTGOING"] = "OUTGOING";
    LocationFilterMode["BETWEEN"] = "BETWEEN";
})(LocationFilterMode || (LocationFilterMode = {}));
export var ActionType;
(function (ActionType) {
    ActionType["SET_VIEWPORT"] = "SET_VIEWPORT";
    ActionType["ZOOM_IN"] = "ZOOM_IN";
    ActionType["ZOOM_OUT"] = "ZOOM_OUT";
    ActionType["RESET_BEARING_PITCH"] = "RESET_BEARING_PITCH";
    ActionType["SET_HIGHLIGHT"] = "SET_HIGHLIGHT";
    ActionType["SET_TOOLTIP"] = "SET_TOOLTIP";
    ActionType["CLEAR_SELECTION"] = "CLEAR_SELECTION";
    ActionType["SELECT_LOCATION"] = "SELECT_LOCATION";
    ActionType["SET_SELECTED_LOCATIONS"] = "SET_SELECTED_LOCATIONS";
    ActionType["SET_LOCATION_FILTER_MODE"] = "SET_LOCATION_FILTER_MODE";
    ActionType["SET_TIME_RANGE"] = "SET_TIME_RANGE";
    ActionType["SET_CLUSTERING_ENABLED"] = "SET_CLUSTERING_ENABLED";
    ActionType["SET_CLUSTERING_AUTO"] = "SET_CLUSTERING_AUTO";
    ActionType["SET_MANUAL_CLUSTER_ZOOM"] = "SET_MANUAL_CLUSTER_ZOOM";
    ActionType["SET_ANIMATION_ENABLED"] = "SET_ANIMATION_ENABLED";
    ActionType["SET_LOCATION_TOTALS_ENABLED"] = "SET_LOCATION_TOTALS_ENABLED";
    ActionType["SET_ADAPTIVE_SCALES_ENABLED"] = "SET_ADAPTIVE_SCALES_ENABLED";
    ActionType["SET_DARK_MODE"] = "SET_DARK_MODE";
    ActionType["SET_FADE_ENABLED"] = "SET_FADE_ENABLED";
    ActionType["SET_BASE_MAP_ENABLED"] = "SET_BASE_MAP_ENABLED";
    ActionType["SET_FADE_AMOUNT"] = "SET_FADE_AMOUNT";
    ActionType["SET_BASE_MAP_OPACITY"] = "SET_BASE_MAP_OPACITY";
    ActionType["SET_COLOR_SCHEME"] = "SET_COLOR_SCHEME";
})(ActionType || (ActionType = {}));
function mainReducer(state, action) {
    switch (action.type) {
        case ActionType.SET_VIEWPORT: {
            var viewport_1 = action.viewport, adjustViewportToLocations = action.adjustViewportToLocations;
            return __assign(__assign(__assign({}, state), { viewport: __assign(__assign({}, viewport_1), { zoom: Math.min(MAX_ZOOM_LEVEL, Math.max(MIN_ZOOM_LEVEL, viewport_1.zoom)) }), tooltip: undefined, highlight: undefined }), (adjustViewportToLocations != null && {
                adjustViewportToLocations: false,
            }));
        }
        case ActionType.ZOOM_IN: {
            var viewport_2 = state.viewport;
            return __assign(__assign({}, state), { viewport: __assign(__assign({}, viewport_2), { zoom: Math.min(MAX_ZOOM_LEVEL, viewport_2.zoom * 1.1) }), tooltip: undefined, highlight: undefined });
        }
        case ActionType.ZOOM_OUT: {
            var viewport_3 = state.viewport;
            return __assign(__assign({}, state), { viewport: __assign(__assign({}, viewport_3), { zoom: Math.max(MIN_ZOOM_LEVEL, viewport_3.zoom / 1.1) }), tooltip: undefined, highlight: undefined });
        }
        case ActionType.RESET_BEARING_PITCH: {
            var viewport_4 = state.viewport;
            return __assign(__assign({}, state), { viewport: __assign(__assign(__assign({}, viewport_4), { bearing: 0, pitch: 0 }), mapTransition(500)) });
        }
        case ActionType.SET_HIGHLIGHT: {
            var highlight = action.highlight;
            return __assign(__assign({}, state), { highlight: highlight });
        }
        case ActionType.SET_TOOLTIP: {
            var tooltip = action.tooltip;
            return __assign(__assign({}, state), { tooltip: tooltip });
        }
        case ActionType.CLEAR_SELECTION: {
            return __assign(__assign({}, state), { selectedLocations: undefined, locationFilterMode: LocationFilterMode.ALL, highlight: undefined, tooltip: undefined });
        }
        case ActionType.SET_SELECTED_LOCATIONS: {
            var selectedLocations = action.selectedLocations;
            var isEmpty = !selectedLocations || selectedLocations.length === 0;
            if (isEmpty) {
                return __assign(__assign({}, state), { locationFilterMode: LocationFilterMode.ALL, selectedLocations: undefined });
            }
            return __assign(__assign({}, state), { selectedLocations: selectedLocations });
        }
        case ActionType.SET_LOCATION_FILTER_MODE: {
            var mode = action.mode;
            return __assign(__assign({}, state), { locationFilterMode: mode });
        }
        case ActionType.SET_TIME_RANGE: {
            var range = action.range;
            return __assign(__assign({}, state), { selectedTimeRange: range });
        }
        case ActionType.SELECT_LOCATION: {
            var selectedLocations = state.selectedLocations;
            var locationId_1 = action.locationId, incremental = action.incremental;
            var nextSelectedLocations = void 0;
            if (selectedLocations) {
                var idx = selectedLocations.findIndex(function (id) { return id === locationId_1; });
                if (idx >= 0) {
                    nextSelectedLocations = selectedLocations.slice();
                    nextSelectedLocations.splice(idx, 1);
                    if (nextSelectedLocations.length === 0)
                        nextSelectedLocations = undefined;
                }
                else {
                    if (incremental) {
                        nextSelectedLocations = __spread(selectedLocations, [locationId_1]);
                    }
                    else {
                        nextSelectedLocations = [locationId_1];
                    }
                }
            }
            else {
                nextSelectedLocations = [locationId_1];
            }
            return __assign(__assign(__assign(__assign({}, state), { selectedLocations: nextSelectedLocations }), (!nextSelectedLocations && {
                locationFilterMode: LocationFilterMode.ALL,
            })), { highlight: undefined, tooltip: undefined });
        }
        case ActionType.SET_CLUSTERING_ENABLED: {
            var clusteringEnabled = action.clusteringEnabled;
            return __assign(__assign({}, state), { clusteringEnabled: clusteringEnabled });
        }
        case ActionType.SET_CLUSTERING_AUTO: {
            var clusteringAuto = action.clusteringAuto;
            return __assign(__assign({}, state), { clusteringAuto: clusteringAuto });
        }
        case ActionType.SET_ANIMATION_ENABLED: {
            var animationEnabled = action.animationEnabled;
            return __assign(__assign({}, state), { animationEnabled: animationEnabled });
        }
        case ActionType.SET_LOCATION_TOTALS_ENABLED: {
            var locationTotalsEnabled = action.locationTotalsEnabled;
            return __assign(__assign({}, state), { locationTotalsEnabled: locationTotalsEnabled });
        }
        case ActionType.SET_ADAPTIVE_SCALES_ENABLED: {
            var adaptiveScalesEnabled = action.adaptiveScalesEnabled;
            return __assign(__assign({}, state), { adaptiveScalesEnabled: adaptiveScalesEnabled });
        }
        case ActionType.SET_DARK_MODE: {
            var darkMode = action.darkMode;
            return __assign(__assign({}, state), { darkMode: darkMode });
        }
        case ActionType.SET_FADE_ENABLED: {
            var fadeEnabled = action.fadeEnabled;
            return __assign(__assign({}, state), { fadeEnabled: fadeEnabled });
        }
        case ActionType.SET_BASE_MAP_ENABLED: {
            var baseMapEnabled = action.baseMapEnabled;
            return __assign(__assign({}, state), { baseMapEnabled: baseMapEnabled });
        }
        case ActionType.SET_FADE_AMOUNT: {
            var fadeAmount = action.fadeAmount;
            return __assign(__assign({}, state), { fadeAmount: fadeAmount });
        }
        case ActionType.SET_BASE_MAP_OPACITY: {
            var baseMapOpacity = action.baseMapOpacity;
            return __assign(__assign({}, state), { baseMapOpacity: baseMapOpacity });
        }
        case ActionType.SET_MANUAL_CLUSTER_ZOOM: {
            var manualClusterZoom = action.manualClusterZoom;
            return __assign(__assign({}, state), { manualClusterZoom: manualClusterZoom });
        }
        case ActionType.SET_COLOR_SCHEME: {
            var colorSchemeKey = action.colorSchemeKey;
            return __assign(__assign({}, state), { colorSchemeKey: colorSchemeKey });
        }
    }
    return state;
}
export var reducer = function (state, action) {
    var nextState = mainReducer(state, action);
    // console.log(action.type, action);
    return nextState;
};
export function asNumber(v) {
    if (typeof v === 'string') {
        var val = +v;
        if (!isNaN(val) && isFinite(val))
            return val;
    }
    return undefined;
}
export function asBoolean(v) {
    if (v === '1' || v === '0') {
        return v === '1';
    }
    return undefined;
}
export function applyStateFromQueryString(draft, query) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l;
    var params = queryString.parse(query.substr(1));
    if (typeof params.s === 'string') {
        var rows = csvParseRows(params.s);
        if (rows.length > 0) {
            draft.selectedLocations = rows[0];
        }
    }
    if (typeof params.v === 'string') {
        var rows = csvParseRows(params.v);
        if (rows.length > 0) {
            var _m = __read(rows[0].map(asNumber), 5), latitude = _m[0], longitude = _m[1], zoom = _m[2], bearing = _m[3], pitch = _m[4];
            if (latitude != null && longitude != null && zoom != null) {
                draft.viewport = __assign(__assign(__assign(__assign({}, draft.viewport), { latitude: latitude,
                    longitude: longitude,
                    zoom: zoom }), (bearing != null ? { bearing: bearing } : undefined)), (pitch != null ? { pitch: pitch } : undefined));
                draft.adjustViewportToLocations = false;
            }
        }
    }
    draft.baseMapOpacity = (_a = asNumber(params.bo)) !== null && _a !== void 0 ? _a : draft.baseMapOpacity;
    draft.manualClusterZoom = (_b = asNumber(params.cz)) !== null && _b !== void 0 ? _b : draft.manualClusterZoom;
    draft.baseMapEnabled = (_c = asBoolean(params.b)) !== null && _c !== void 0 ? _c : draft.baseMapEnabled;
    draft.darkMode = (_d = asBoolean(params.d)) !== null && _d !== void 0 ? _d : draft.darkMode;
    draft.fadeEnabled = (_e = asBoolean(params.fe)) !== null && _e !== void 0 ? _e : draft.fadeEnabled;
    draft.fadeAmount = (_f = asNumber(params.f)) !== null && _f !== void 0 ? _f : draft.fadeAmount;
    draft.animationEnabled = (_g = asBoolean(params.a)) !== null && _g !== void 0 ? _g : draft.animationEnabled;
    draft.adaptiveScalesEnabled = (_h = asBoolean(params.as)) !== null && _h !== void 0 ? _h : draft.adaptiveScalesEnabled;
    draft.clusteringEnabled = (_j = asBoolean(params.c)) !== null && _j !== void 0 ? _j : draft.clusteringEnabled;
    draft.clusteringAuto = (_k = asBoolean(params.ca)) !== null && _k !== void 0 ? _k : draft.clusteringAuto;
    draft.locationTotalsEnabled = (_l = asBoolean(params.lt)) !== null && _l !== void 0 ? _l : draft.locationTotalsEnabled;
    if (params.lfm != null && params.lfm in LocationFilterMode) {
        draft.locationFilterMode = params.lfm;
    }
    if (typeof params.t === 'string') {
        var parts = params.t.split(',');
        var t1 = timeFromQuery(parts[0]);
        var t2 = timeFromQuery(parts[1]);
        if (t1 && t2) {
            draft.selectedTimeRange = [t1, t2];
        }
    }
    if (typeof params.col === 'string' && COLOR_SCHEME_KEYS.includes(params.col)) {
        draft.colorSchemeKey = params.col;
    }
}
export function stateToQueryString(state) {
    var parts = [];
    var _a = state.viewport, latitude = _a.latitude, longitude = _a.longitude, zoom = _a.zoom, bearing = _a.bearing, pitch = _a.pitch, selectedLocations = state.selectedLocations;
    parts.push("v=" + csvFormatRows([
        [
            latitude.toFixed(6),
            longitude.toFixed(6),
            zoom.toFixed(2),
            bearing.toFixed(0),
            pitch.toFixed(0),
        ],
    ]));
    parts.push("a=" + (state.animationEnabled ? 1 : 0));
    parts.push("as=" + (state.adaptiveScalesEnabled ? 1 : 0));
    parts.push("b=" + (state.baseMapEnabled ? 1 : 0));
    parts.push("bo=" + state.baseMapOpacity);
    parts.push("c=" + (state.clusteringEnabled ? 1 : 0));
    parts.push("ca=" + (state.clusteringAuto ? 1 : 0));
    if (state.manualClusterZoom != null)
        parts.push("cz=" + state.manualClusterZoom);
    parts.push("d=" + (state.darkMode ? 1 : 0));
    parts.push("fe=" + (state.fadeEnabled ? 1 : 0));
    parts.push("lt=" + (state.locationTotalsEnabled ? 1 : 0));
    parts.push("lfm=" + state.locationFilterMode);
    if (state.selectedTimeRange) {
        parts.push("t=" + state.selectedTimeRange.map(timeToQuery));
    }
    if (state.colorSchemeKey != null) {
        parts.push("col=" + state.colorSchemeKey);
    }
    parts.push("f=" + state.fadeAmount);
    if (selectedLocations) {
        parts.push("s=" + encodeURIComponent(csvFormatRows([selectedLocations])));
    }
    return parts.join('&');
}
export function getInitialViewport(bbox) {
    var width = window.innerWidth;
    var height = window.innerHeight;
    var _a = viewport(bbox, [width, height], undefined, undefined, 512, true), _b = __read(_a.center, 2), longitude = _b[0], latitude = _b[1], zoom = _a.zoom;
    return {
        width: width,
        height: height,
        longitude: longitude,
        latitude: latitude,
        zoom: zoom,
        minZoom: MIN_ZOOM_LEVEL,
        maxZoom: MAX_ZOOM_LEVEL,
        minPitch: MIN_PITCH,
        maxPitch: MAX_PITCH,
        bearing: 0,
        pitch: 0,
        altitude: 1.5,
    };
}
export var DEFAULT_VIEWPORT = getInitialViewport([-180, -70, 180, 70]);
export function getInitialState(config, queryString) {
    var draft = {
        viewport: DEFAULT_VIEWPORT,
        adjustViewportToLocations: true,
        selectedLocations: undefined,
        locationTotalsEnabled: true,
        locationFilterMode: LocationFilterMode.ALL,
        baseMapEnabled: true,
        adaptiveScalesEnabled: true,
        animationEnabled: parseBoolConfigProp(config[ConfigPropName.ANIMATE_FLOWS]),
        clusteringEnabled: parseBoolConfigProp(config[ConfigPropName.CLUSTER_ON_ZOOM] || 'true'),
        manualClusterZoom: undefined,
        fadeEnabled: true,
        clusteringAuto: true,
        darkMode: parseBoolConfigProp(config[ConfigPropName.COLORS_DARK_MODE] || 'true'),
        fadeAmount: parseNumberConfigProp(config[ConfigPropName.FADE_AMOUNT], 50),
        baseMapOpacity: parseNumberConfigProp(config[ConfigPropName.BASE_MAP_OPACITY], 75),
        colorSchemeKey: config[ConfigPropName.COLORS_SCHEME],
        selectedFlowsSheet: undefined,
        selectedTimeRange: undefined,
    };
    var bbox = config[ConfigPropName.MAP_BBOX];
    if (bbox) {
        var bounds = bbox
            .split(',')
            .map(asNumber)
            .filter(function (v) { return v != null; });
        if (bounds.length === 4) {
            draft.viewport = getInitialViewport(bounds);
            draft.adjustViewportToLocations = false;
        }
    }
    if (queryString && queryString.length > 1) {
        applyStateFromQueryString(draft, queryString);
    }
    return draft;
}
//# sourceMappingURL=FlowMap.state.js.map