var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
import { Classes, Intent, MenuItem } from '@blueprintjs/core';
import { nest } from 'd3-collection';
import React from 'react';
import { defaultMemoize } from 'reselect';
import { matchesSearchQuery } from './matchesSearchQuery';
import SearchBox from './SearchBox';
import styled from '@emotion/styled';
var LocationTag = styled.div({
    display: 'flex',
    fontSize: 10,
    alignItems: 'center',
    '& > * + *': {
        marginLeft: 5,
    },
});
var itemPredicate = function (query, location) {
    var id = location.id, name = location.name;
    return matchesSearchQuery(query, id + " " + name);
};
function sortLocations(locations) {
    return locations.slice().sort(function (a, b) {
        var aname = a.name || a.id;
        var bname = b.name || b.id;
        if (aname < bname)
            return -1;
        if (aname > bname)
            return 1;
        return 0;
    });
}
function getSelectedLocationsSet(selectedLocations) {
    if (!selectedLocations || selectedLocations.length === 0) {
        return undefined;
    }
    return new Set(selectedLocations);
}
function getLocationsBySelectionStatus(locations, selectedLocations) {
    var selectedIds = getSelectedLocationsSet(selectedLocations);
    if (!selectedIds) {
        return {
            selected: undefined,
            unselected: locations,
        };
    }
    var _a = nest()
        .key(function (location) { return (selectedIds.has(location.id) ? 'selected' : 'unselected'); })
        .object(locations), selected = _a.selected, unselected = _a.unselected;
    return {
        selected: selected,
        unselected: unselected,
    };
}
var Outer = styled.div(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n  & > * > .", " {\n    width: 15rem;\n  }\n  .", " {\n    border-top-left-radius: 5px;\n    border-top-right-radius: 5px;\n  }\n  .", " {\n    max-height: 150px;\n    overflow-y: auto;\n    align-items: flex-start;\n  }\n  .", " {\n    & > select {\n      font-size: small;\n    }\n  }\n"], ["\n  & > * > .", " {\n    width: 15rem;\n  }\n  .", " {\n    border-top-left-radius: 5px;\n    border-top-right-radius: 5px;\n  }\n  .", " {\n    max-height: 150px;\n    overflow-y: auto;\n    align-items: flex-start;\n  }\n  .", " {\n    & > select {\n      font-size: small;\n    }\n  }\n"])), Classes.POPOVER_TARGET, Classes.TAG_INPUT, Classes.TAG_INPUT_VALUES, Classes.HTML_SELECT);
var TextOverflowEllipsis = styled.span({
    textOverflow: 'ellipsis',
    whiteSpace: 'nowrap',
    overflow: 'hidden',
    maxWidth: 180,
});
var LocationsSearchBox = /** @class */ (function (_super) {
    __extends(LocationsSearchBox, _super);
    function LocationsSearchBox() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.getSortedLocations = defaultMemoize(sortLocations);
        _this.getLocationsBySelectionStatus = defaultMemoize(getLocationsBySelectionStatus);
        _this.tagRenderer = function (location) {
            var selectedLocations = _this.props.selectedLocations;
            var selection = selectedLocations && selectedLocations.find(function (id) { return id === location.id; });
            if (!selection) {
                return null;
            }
            return (React.createElement(LocationTag, null,
                React.createElement(TextOverflowEllipsis, null, location.name)));
        };
        _this.itemRenderer = function (item, _a) {
            var handleClick = _a.handleClick, modifiers = _a.modifiers;
            if (!modifiers.matchesPredicate) {
                return null;
            }
            var _b = item, id = _b.id, name = _b.name;
            var selectedLocations = _this.props.selectedLocations;
            var isSelected = selectedLocations && selectedLocations.indexOf(id) >= 0;
            var intent = isSelected ? Intent.PRIMARY : Intent.NONE;
            return (React.createElement(MenuItem, { key: id, active: modifiers.active, text: name, intent: intent, onClick: handleClick }));
        };
        _this.handleSelectionCleared = function () { return _this.props.onSelectionChanged(undefined); };
        _this.handleLocationSelected = function (location) {
            var _a = _this.props, selectedLocations = _a.selectedLocations, onSelectionChanged = _a.onSelectionChanged;
            var id = location.id;
            if (selectedLocations) {
                if (selectedLocations.indexOf(id) < 0) {
                    onSelectionChanged(__spread(selectedLocations, [id]));
                }
            }
            else {
                onSelectionChanged([id]);
            }
        };
        _this.handleLocationRemoved = function (location) {
            var _a = _this.props, selectedLocations = _a.selectedLocations, onSelectionChanged = _a.onSelectionChanged;
            if (selectedLocations) {
                var id = location.id;
                var idx = selectedLocations.indexOf(id);
                if (idx >= 0) {
                    var next = selectedLocations.slice();
                    next.splice(idx, 1);
                    onSelectionChanged(selectedLocations.length === 1 ? undefined : next);
                }
            }
        };
        return _this;
    }
    LocationsSearchBox.prototype.render = function () {
        var _a = this.props, locations = _a.locations, selectedLocations = _a.selectedLocations, locationFilterMode = _a.locationFilterMode, onLocationFilterModeChange = _a.onLocationFilterModeChange;
        var _b = this.getLocationsBySelectionStatus(this.getSortedLocations(locations), selectedLocations), selected = _b.selected, unselected = _b.unselected;
        return (React.createElement(Outer, null,
            React.createElement(SearchBox, { placeholder: "Filter by origin and destination\u2026", items: unselected, selectedItems: selected, maxItems: 100, itemPredicate: itemPredicate, itemRenderer: this.itemRenderer, tagRenderer: this.tagRenderer, onCleared: this.handleSelectionCleared, onRemoved: this.handleLocationRemoved, onSelected: this.handleLocationSelected, locationFilterMode: locationFilterMode, onLocationFilterModeChange: onLocationFilterModeChange })));
    };
    return LocationsSearchBox;
}(React.PureComponent));
export default LocationsSearchBox;
var templateObject_1;
//# sourceMappingURL=LocationSearchBox.js.map