var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var _a, _b, _c, _d;
import React, { useCallback, useEffect } from 'react';
import { Editor, DrawPolygonMode, RENDER_STATE } from 'react-map-gl-draw';
import { Colors } from '@blueprintjs/core';
export var MapDrawingMode;
(function (MapDrawingMode) {
    MapDrawingMode["POLYGON"] = "POLYGON";
})(MapDrawingMode || (MapDrawingMode = {}));
export var HANDLE_RADIUS = 5;
export var HANDLE_STROKE = (_a = {
        DEFAULT: Colors.WHITE
    },
    _a[RENDER_STATE.SELECTED] = Colors.BLUE3,
    _a[RENDER_STATE.HOVERED] = Colors.VERMILION3,
    _a);
export var HANDLE_FILL = (_b = {
        DEFAULT: Colors.WHITE
    },
    _b[RENDER_STATE.SELECTED] = Colors.BLUE3,
    _b[RENDER_STATE.HOVERED] = Colors.BLUE5,
    _b[RENDER_STATE.INACTIVE] = Colors.GRAY2,
    _b[RENDER_STATE.UNCOMMITTED] = Colors.GRAY2,
    _b);
function getEditHandleStyle(_a) {
    var feature = _a.feature, state = _a.state;
    return {
        stroke: HANDLE_STROKE[state] || HANDLE_STROKE.DEFAULT,
        strokeWidth: 2,
        fill: HANDLE_FILL[state] || HANDLE_FILL.DEFAULT,
        fillOpacity: 1.0,
        r: HANDLE_RADIUS,
    };
}
export var FEATURE_STROKE = (_c = {
        DEFAULT: Colors.ORANGE5
    },
    _c[RENDER_STATE.INACTIVE] = Colors.ORANGE5,
    _c[RENDER_STATE.UNCOMMITTED] = Colors.ORANGE3,
    _c[RENDER_STATE.CLOSING] = Colors.ORANGE3,
    _c[RENDER_STATE.SELECTED] = Colors.ORANGE5,
    _c[RENDER_STATE.HOVERED] = Colors.ORANGE5,
    _c);
export var FEATURE_FILL = (_d = {
        DEFAULT: Colors.ORANGE5
    },
    _d[RENDER_STATE.INACTIVE] = Colors.ORANGE5,
    _d[RENDER_STATE.HOVERED] = Colors.ORANGE3,
    _d[RENDER_STATE.SELECTED] = Colors.ORANGE5,
    _d[RENDER_STATE.UNCOMMITTED] = Colors.ORANGE4,
    _d[RENDER_STATE.CLOSING] = Colors.GRAY1,
    _d);
function getFeatureStyle(_a) {
    var feature = _a.feature, state = _a.state;
    return __assign({ stroke: FEATURE_STROKE[state] || FEATURE_STROKE.DEFAULT, strokeWidth: 2, fill: FEATURE_FILL[state] || FEATURE_FILL.DEFAULT, fillOpacity: 0.2 }, (state !== RENDER_STATE.SELECTED ? { strokeDasharray: '4,2' } : null));
}
var DRAW_POLYGON_MODE = new DrawPolygonMode();
function getEditorMode(mapDrawingMode) {
    switch (mapDrawingMode) {
        case MapDrawingMode.POLYGON:
            return DRAW_POLYGON_MODE;
    }
    return undefined;
}
var MapDrawingEditor = function (_a) {
    var mapDrawingMode = _a.mapDrawingMode, onFeatureDrawn = _a.onFeatureDrawn;
    var cancelDrawing = useCallback(function () {
        onFeatureDrawn(undefined);
    }, [onFeatureDrawn]);
    useEffect(function () {
        function handleKeyDown(e) {
            if (e instanceof KeyboardEvent && e.code === 'Escape') {
                cancelDrawing();
            }
        }
        document.addEventListener('keydown', handleKeyDown);
        return function () { return document.removeEventListener('keydown', handleKeyDown); };
    }, [cancelDrawing]);
    var handleUpdate = function (info) {
        if (info.editType === 'addFeature') {
            var data = info.data;
            if (data.length > 0) {
                onFeatureDrawn(data[data.length - 1]);
            }
            else {
                onFeatureDrawn(undefined);
            }
        }
    };
    return (React.createElement(Editor, { clickRadius: HANDLE_RADIUS, mode: getEditorMode(mapDrawingMode), editHandleShape: "circle", onUpdate: handleUpdate, featureStyle: getFeatureStyle, editHandleStyle: getEditHandleStyle }));
};
export default MapDrawingEditor;
//# sourceMappingURL=MapDrawingEditor.js.map