var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
import React, { useEffect, useRef, useState } from 'react';
import styled from '@emotion/styled';
import { scaleLinear, scaleTime } from 'd3-scale';
import { max } from 'd3-array';
import { EventManager } from 'mjolnir.js';
import PlayControl from './PlayControl';
import { Colors } from '@blueprintjs/core';
import { useMeasure, useThrottle } from 'react-use';
import { areRangesEqual, tickMultiFormat } from './time';
import { ColorScheme } from './colors';
import { hcl } from 'd3-color';
var SVG_HEIGHT = 80;
var AXIS_AREA_HEIGHT = 20;
var TOTAL_COUNT_CHART_HEIGHT = 30;
var margin = {
    top: 15,
    left: 25,
    right: 25,
    bottom: 15,
};
var Outer = styled.div({
    display: 'flex',
    padding: '5px 20px',
    alignItems: 'center',
    userSelect: 'none',
    '&>*+*': {
        marginLeft: 10,
    },
});
var MeasureTarget = styled.div({
    display: 'flex',
    flexGrow: 1,
    overflow: 'hidden',
});
var TimelineSvg = styled.svg(function (props) { return ({
    cursor: 'pointer',
    backgroundColor: props.darkMode ? Colors.DARK_GRAY4 : Colors.LIGHT_GRAY4,
}); });
var OuterRect = styled.rect({
    cursor: 'crosshair',
    fill: 'rgba(255,255,255,0)',
    stroke: 'none',
});
var HandleOuter = styled.g(function (props) { return ({
    cursor: 'ew-resize',
    '& > path': {
        stroke: Colors.DARK_GRAY1,
        transition: 'fill 0.2s',
        fill: props.darkMode ? Colors.GRAY4 : Colors.WHITE,
    },
    '&:hover path': {
        fill: props.darkMode ? Colors.WHITE : Colors.GRAY4,
    },
}); });
var HandlePath = styled.path({
    strokeWidth: 1,
});
var HandleHoverTarget = styled.rect({
    fill: Colors.WHITE,
    fillOpacity: 0,
});
var SelectedRangeRect = styled.rect({
    fill: Colors.BLUE5,
    cursor: 'move',
    transition: 'fill-opacity 0.2s',
    fillOpacity: 0.3,
    '&:hover': {
        fillOpacity: 0.4,
    },
});
var AxisPath = styled.path({
    fill: 'none',
    stroke: Colors.GRAY1,
    shapeRendering: 'crispEdges',
});
var TickText = styled.text(function (props) { return ({
    fill: props.darkMode ? Colors.LIGHT_GRAY1 : Colors.DARK_GRAY1,
    fontSize: 10,
    textAnchor: 'start',
}); });
var Bar = styled.rect(function (props) { return ({
    fill: props.darkMode ? Colors.LIGHT_GRAY1 : ColorScheme.primary,
    stroke: props.darkMode ? Colors.GRAY4 : hcl(ColorScheme.primary).darker().toString(),
}); });
var TickLine = styled.line({
    fill: 'none',
    stroke: Colors.GRAY1,
    shapeRendering: 'crispEdges',
});
var TimelineHandle = function (props) {
    var width = props.width, height = props.height, side = props.side, darkMode = props.darkMode, onMove = props.onMove;
    var handleMoveRef = useRef();
    handleMoveRef.current = function (_a) {
        var center = _a.center;
        onMove(center.x, side);
    };
    var ref = useRef(null);
    useEffect(function () {
        var eventManager = new EventManager(ref.current);
        if (handleMoveRef.current) {
            eventManager.on('panstart', handleMoveRef.current);
            eventManager.on('panmove', handleMoveRef.current);
            eventManager.on('panend', handleMoveRef.current);
        }
        return function () {
            // eventManager.off('panstart', handleMove);
            // eventManager.off('panmove', handleMove);
            // eventManager.off('panend', handleMove);
            eventManager.destroy();
        };
    }, [handleMoveRef]);
    var _a = __read([width, width], 2), w = _a[0], h = _a[1];
    return (React.createElement(HandleOuter, { darkMode: darkMode },
        React.createElement(HandlePath, { transform: "translate(" + (side === 'start' ? 0 : width) + ",0)", d: side === 'start'
                ? "M0," + h + " Q" + -w + "," + h + " " + -w + ",0 L0,0 0," + height + " " + -w + "," + height + " Q" + -w + "," + (height - h) + " 0," + (height - h) + " z"
                : "M0," + h + " Q" + w + "," + h + " " + w + ",0 L0,0 0," + height + " " + w + "," + height + " Q" + w + "," + (height - h) + " 0," + (height - h) + " z" }),
        React.createElement(HandleHoverTarget, { x: side === 'start' ? -w : w, ref: ref, height: height, width: width })));
};
var TimelineChart = function (props) {
    var _a;
    var width = props.width, extent = props.extent, selectedRange = props.selectedRange, timeGranularity = props.timeGranularity, totalCountsByTime = props.totalCountsByTime, darkMode = props.darkMode, onChange = props.onChange;
    var handleWidth = 10;
    var handleHGap = 10;
    var handleHeight = TOTAL_COUNT_CHART_HEIGHT + AXIS_AREA_HEIGHT + handleHGap * 2;
    var chartWidth = width - margin.left - margin.right;
    var timeScale = scaleTime();
    timeScale.domain(extent);
    timeScale.range([0, chartWidth]);
    var _b = __read(useState(), 2), offset = _b[0], setOffset = _b[1];
    var _c = __read(useState(), 2), panStart = _c[0], setPanStart = _c[1];
    var svgRef = useRef(null);
    var outerRectRef = useRef(null);
    var selectedRangeRectRef = useRef(null);
    var mousePosition = function (absPos) {
        var current = svgRef.current;
        if (current != null) {
            var left = current.getBoundingClientRect().left;
            return absPos - left - margin.left;
        }
        return undefined;
    };
    var timeFromPos = function (pos) {
        var relPos = mousePosition(pos);
        if (relPos != null)
            return timeScale.invert(relPos);
        return undefined;
    };
    var handleMoveRef = useRef();
    handleMoveRef.current = function (_a) {
        var center = _a.center;
        if (offset == null) {
            var pos = mousePosition(center.x);
            if (pos != null) {
                setOffset(timeScale(selectedRange[0]) - pos);
            }
        }
        else {
            var nextStart = timeFromPos(center.x + offset);
            if (nextStart) {
                var interval = timeGranularity.interval;
                nextStart = interval.round(nextStart);
                var length_1 = interval.count(selectedRange[0], selectedRange[1]);
                var nextEnd = interval.offset(nextStart, length_1);
                if (nextStart < extent[0]) {
                    nextStart = extent[0];
                    nextEnd = interval.offset(extent[0], length_1);
                }
                if (nextEnd > extent[1]) {
                    nextStart = interval.offset(extent[1], -length_1);
                    nextEnd = extent[1];
                }
                onChange([nextStart, nextEnd]);
            }
        }
    };
    var handleMove = function (evt) { return handleMoveRef.current(evt); };
    var handleMoveEnd = function (evt) {
        handleMoveRef.current(evt);
        setOffset(undefined);
    };
    var handleClickRef = useRef();
    handleClickRef.current = function (evt) {
        onChange(extent);
    };
    var handleClick = function (evt) { return handleClickRef.current(evt); };
    var handlePanStartRef = useRef();
    handlePanStartRef.current = function (_a) {
        var center = _a.center;
        var start = timeFromPos(center.x);
        if (start) {
            start = timeGranularity.interval.round(start);
            if (start < extent[0])
                start = extent[0];
            if (start > extent[1])
                start = extent[1];
            setPanStart(start);
            onChange([start, start]);
        }
    };
    var handlePanStart = function (evt) { return handlePanStartRef.current(evt); };
    var handlePanMoveRef = useRef();
    handlePanMoveRef.current = function (_a) {
        var center = _a.center;
        var end = timeFromPos(center.x);
        if (panStart && end) {
            end = timeGranularity.interval.round(end);
            if (end < extent[0])
                end = extent[0];
            if (end > extent[1])
                end = extent[1];
            var range = panStart < end ? [panStart, end] : [end, panStart];
            onChange(range);
        }
    };
    var handlePanMove = function (evt) { return handlePanMoveRef.current(evt); };
    var handlePanEnd = function (evt) {
        handlePanMoveRef.current(evt);
        setPanStart(undefined);
    };
    useEffect(function () {
        var outerEvents = new EventManager(outerRectRef.current);
        outerEvents.on('click', handleClick);
        outerEvents.on('panstart', handlePanStart);
        outerEvents.on('panmove', handlePanMove);
        outerEvents.on('panend', handlePanEnd);
        var selectedRangeEvents = new EventManager(selectedRangeRectRef.current);
        selectedRangeEvents.on('panstart', handleMove);
        selectedRangeEvents.on('panmove', handleMove);
        selectedRangeEvents.on('panend', handleMoveEnd);
        return function () {
            outerEvents.destroy();
            selectedRangeEvents.destroy();
        };
    }, []);
    var handleMoveSideRef = useRef();
    handleMoveSideRef.current = function (pos, side) {
        var t = timeFromPos(pos);
        if (t != null) {
            t = timeGranularity.interval.round(t);
            if (t < extent[0])
                t = extent[0];
            if (t > extent[1])
                t = extent[1];
            if (side === 'start') {
                onChange([t < selectedRange[1] ? t : selectedRange[1], selectedRange[1]]);
            }
            else {
                onChange([selectedRange[0], t > selectedRange[0] ? t : selectedRange[0]]);
            }
        }
    };
    var handleMoveSide = function (pos, kind) {
        if (handleMoveSideRef.current) {
            handleMoveSideRef.current(pos, kind);
        }
    };
    var minLabelWidth = 70;
    var ticks = timeScale.ticks(Math.min(timeGranularity.interval.count(extent[0], extent[1]), chartWidth / minLabelWidth));
    var totalCountScale = scaleLinear()
        .domain([0, (_a = max(totalCountsByTime, function (d) { return d.count; })) !== null && _a !== void 0 ? _a : 0])
        .range([0, TOTAL_COUNT_CHART_HEIGHT]);
    var tickLabelFormat = tickMultiFormat; // timeScale.tickFormat();
    return (React.createElement(TimelineSvg, { ref: svgRef, darkMode: darkMode, width: width, height: SVG_HEIGHT },
        React.createElement("g", { transform: "translate(" + margin.left + "," + margin.top + ")" },
            ticks.map(function (t, i) { return (React.createElement("g", { key: i, transform: "translate(" + timeScale(t) + "," + 0 + ")" },
                React.createElement(TickLine, { y1: 0, y2: TOTAL_COUNT_CHART_HEIGHT + AXIS_AREA_HEIGHT }),
                timeScale(t) < chartWidth &&
                    React.createElement(TickText, { darkMode: darkMode, x: 3, y: 12 }, 
                    // timeGranularity.format(t)
                    tickLabelFormat(t)))); }),
            React.createElement(AxisPath, { d: "M0,0 " + chartWidth + ",0" }),
            React.createElement(AxisPath, { transform: "translate(0," + (TOTAL_COUNT_CHART_HEIGHT + AXIS_AREA_HEIGHT) + ")", d: "M0,0 " + chartWidth + ",0" }),
            React.createElement("g", { transform: "translate(0," + AXIS_AREA_HEIGHT + ")" }, totalCountsByTime.map(function (_a) {
                var time = _a.time, count = _a.count;
                return (React.createElement(Bar, { darkMode: darkMode, key: time.getTime(), x: timeScale(time), y: TOTAL_COUNT_CHART_HEIGHT - totalCountScale(count), width: Math.max(timeScale(timeGranularity.interval.offset(time)) - timeScale(time) - 1, 1), height: totalCountScale(count) }));
            }))),
        React.createElement(OuterRect, { ref: outerRectRef, width: width, height: SVG_HEIGHT }),
        React.createElement("g", { transform: "translate(" + margin.left + "," + margin.top + ")" },
            React.createElement("g", { transform: "translate(" + timeScale(selectedRange[0]) + ",0)" },
                React.createElement(SelectedRangeRect, { ref: selectedRangeRectRef, height: TOTAL_COUNT_CHART_HEIGHT + AXIS_AREA_HEIGHT, width: timeScale(selectedRange[1]) - timeScale(selectedRange[0]) })),
            React.createElement("g", { transform: "translate(" + timeScale(selectedRange[0]) + "," + -handleHGap + ")" },
                React.createElement(TimelineHandle, { darkMode: darkMode, width: handleWidth, height: handleHeight, side: "start", onMove: handleMoveSide })),
            React.createElement("g", { transform: "translate(" + (timeScale(selectedRange[1]) - handleWidth) + "," + -handleHGap + ")" },
                React.createElement(TimelineHandle, { darkMode: darkMode, width: handleWidth, height: handleHeight, side: "end", onMove: handleMoveSide })))));
};
var Timeline = function (props) {
    var _a = __read(useMeasure(), 2), measureRef = _a[0], dimensions = _a[1];
    var extent = props.extent, selectedRange = props.selectedRange, timeGranularity = props.timeGranularity, darkMode = props.darkMode, onChange = props.onChange;
    var _b = __read(useState(selectedRange), 2), internalRange = _b[0], setInternalRange = _b[1];
    var throttledRange = useThrottle(internalRange, 100);
    var onChangeRef = useRef();
    onChangeRef.current = function (range) { return onChange(range); };
    useEffect(function () {
        var current = onChangeRef.current;
        if (current)
            current(throttledRange);
    }, [throttledRange, onChangeRef]);
    var _c = __read(useState(selectedRange), 2), prevSelectedRange = _c[0], setPrevSelectedRange = _c[1];
    if (!areRangesEqual(selectedRange, prevSelectedRange)) {
        setInternalRange(selectedRange);
        setPrevSelectedRange(selectedRange);
    }
    var _d = __read(useState(false), 2), isPlaying = _d[0], setPlaying = _d[1];
    var handlePlay = function () {
        var interval = timeGranularity.interval;
        if (selectedRange[1] >= extent[1]) {
            var length_2 = interval.count(selectedRange[0], selectedRange[1]);
            setInternalRange([extent[0], interval.offset(extent[0], length_2)]);
        }
        setPlaying(true);
    };
    var handlePlayAdvance = function (start) {
        var interval = timeGranularity.interval;
        var length = interval.count(selectedRange[0], selectedRange[1]);
        var end = interval.offset(start, length);
        if (end >= extent[1]) {
            setPlaying(false);
            setInternalRange([interval.offset(end, -length), end]);
        }
        else {
            setInternalRange([start, end]);
        }
    };
    var handleMove = function (range) {
        setInternalRange(range);
        setPlaying(false);
    };
    return (React.createElement(Outer, null,
        React.createElement(PlayControl, { darkMode: darkMode, extent: extent, current: internalRange[0], interval: timeGranularity.interval, stepDuration: 100, speed: 1, isPlaying: isPlaying, onPlay: handlePlay, onPause: function () { return setPlaying(false); }, onAdvance: handlePlayAdvance }),
        React.createElement(MeasureTarget, { ref: measureRef }, dimensions.width > 0 && (React.createElement(TimelineChart, __assign({}, props, { selectedRange: internalRange, width: dimensions.width, onChange: handleMove }))))));
};
export default Timeline;
//# sourceMappingURL=Timeline.js.map