var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
import React from 'react';
import { nest } from 'd3-collection';
import { parseTime } from './time';
import { AppToaster } from './AppToaster';
import { Intent } from '@blueprintjs/core';
import { IconNames } from '@blueprintjs/icons';
import { ToastContent } from './Boxes';
import { ErrorsLocationsBlock, MAX_NUM_OF_IDS_IN_ERROR } from './FlowMap';
export function prepareFlows(rows) {
    var e_1, _a, e_2, _b, e_3, _c;
    var dupes = [];
    // Will sum up duplicate flows (with same origin, dest and time)
    var byOriginDestTime = nest()
        .key(function (d) { return d.origin; })
        .key(function (d) { return d.dest; })
        .key(function (d) { var _a; return ((_a = parseTime(d.time)) === null || _a === void 0 ? void 0 : _a.toISOString()) || 'unknown'; })
        .rollup(function (dd) {
        var _a = dd[0], origin = _a.origin, dest = _a.dest, time = _a.time;
        if (dd.length > 1) {
            dupes.push(dd[0]);
        }
        return {
            origin: origin,
            dest: dest,
            time: parseTime(time),
            count: dd.reduce(function (m, d) {
                if (d.count != null) {
                    var c = +d.count;
                    if (!isNaN(c) && isFinite(c))
                        return m + c;
                }
                return m;
            }, 0),
        };
    })
        .entries(rows);
    var rv = [];
    try {
        for (var byOriginDestTime_1 = __values(byOriginDestTime), byOriginDestTime_1_1 = byOriginDestTime_1.next(); !byOriginDestTime_1_1.done; byOriginDestTime_1_1 = byOriginDestTime_1.next()) {
            var byDestTime = byOriginDestTime_1_1.value;
            try {
                for (var _d = (e_2 = void 0, __values(byDestTime.values)), _e = _d.next(); !_e.done; _e = _d.next()) {
                    var byTime = _e.value;
                    try {
                        for (var _f = (e_3 = void 0, __values(byTime.values)), _g = _f.next(); !_g.done; _g = _f.next()) {
                            var value = _g.value.value;
                            if (value.origin != null && value.dest != null) {
                                rv.push(value);
                            }
                        }
                    }
                    catch (e_3_1) { e_3 = { error: e_3_1 }; }
                    finally {
                        try {
                            if (_g && !_g.done && (_c = _f.return)) _c.call(_f);
                        }
                        finally { if (e_3) throw e_3.error; }
                    }
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (_e && !_e.done && (_b = _d.return)) _b.call(_d);
                }
                finally { if (e_2) throw e_2.error; }
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (byOriginDestTime_1_1 && !byOriginDestTime_1_1.done && (_a = byOriginDestTime_1.return)) _a.call(byOriginDestTime_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    if (dupes.length > 0) {
        AppToaster.show({
            intent: Intent.WARNING,
            icon: IconNames.WARNING_SIGN,
            timeout: 0,
            message: (React.createElement(ToastContent, null,
                "The following flows (origin \u2192 dest pairs) were encountered more than once in the dataset:",
                React.createElement(ErrorsLocationsBlock, null,
                    (dupes.length > MAX_NUM_OF_IDS_IN_ERROR
                        ? dupes.slice(0, MAX_NUM_OF_IDS_IN_ERROR)
                        : dupes)
                        .map(function (_a) {
                        var origin = _a.origin, dest = _a.dest;
                        return origin + " \u2192 " + dest;
                    })
                        .join(', '),
                    dupes.length > MAX_NUM_OF_IDS_IN_ERROR &&
                        " \u2026 and " + (dupes.length - MAX_NUM_OF_IDS_IN_ERROR) + " others"),
                "Their counts were summed up.")),
        });
    }
    return rv;
}
//# sourceMappingURL=prepareFlows.js.map