var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
import { timeFormat, timeParse } from 'd3-time-format';
import { timeDay, timeHour, timeMinute, timeMonth, timeSecond, timeWeek, timeYear, } from 'd3-time';
var dateParsers = [
    timeParse('%Y-%m-%d'),
    timeParse('%Y-%m-%d %H:%M'),
    timeParse('%Y-%m-%d %H:%M:%S'),
    timeParse('%Y'),
    timeParse('%Y-%m'),
];
export function parseTime(input) {
    var e_1, _a;
    if (input != null) {
        if (input instanceof Date) {
            return input;
        }
        try {
            for (var dateParsers_1 = __values(dateParsers), dateParsers_1_1 = dateParsers_1.next(); !dateParsers_1_1.done; dateParsers_1_1 = dateParsers_1.next()) {
                var parse = dateParsers_1_1.value;
                var date = parse(input);
                if (date) {
                    return date;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (dateParsers_1_1 && !dateParsers_1_1.done && (_a = dateParsers_1.return)) _a.call(dateParsers_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    }
    return undefined;
}
export var TimeGranularityKey;
(function (TimeGranularityKey) {
    TimeGranularityKey["SECOND"] = "SECOND";
    TimeGranularityKey["MINUTE"] = "MINUTE";
    TimeGranularityKey["HOUR"] = "HOUR";
    TimeGranularityKey["DAY"] = "DAY";
    TimeGranularityKey["MONTH"] = "MONTH";
    TimeGranularityKey["YEAR"] = "YEAR";
})(TimeGranularityKey || (TimeGranularityKey = {}));
// const preferredLocale = navigator.languages ? navigator.languages[0] : 'en';
var formatMillisecond = timeFormat('.%L'), formatSecond = timeFormat(':%S'), formatMinute = timeFormat('%I:%M'), 
// formatHour = (d: Date) => d.toLocaleString(preferredLocale, { hour: 'numeric' }),
formatHour = timeFormat('%I %p'), formatDay = timeFormat('%a %d'), formatWeek = timeFormat('%b %d'), formatMonth = timeFormat('%b'), formatYear = timeFormat('%Y');
export function tickMultiFormat(date) {
    return (timeSecond(date) < date
        ? formatMillisecond
        : timeMinute(date) < date
            ? formatSecond
            : timeHour(date) < date
                ? formatMinute
                : timeDay(date) < date
                    ? formatHour
                    : timeMonth(date) < date
                        ? timeWeek(date) < date
                            ? formatDay
                            : formatWeek
                        : timeYear(date) < date
                            ? formatMonth
                            : formatYear)(date);
}
export var TIME_GRANULARITIES = [
    {
        order: 0,
        key: TimeGranularityKey.SECOND,
        interval: timeSecond,
        format: formatSecond,
        formatFull: timeFormat('%Y-%m-%d %H:%M:%S'),
    },
    {
        order: 1,
        key: TimeGranularityKey.MINUTE,
        interval: timeMinute,
        format: formatMinute,
        formatFull: timeFormat('%Y-%m-%d %H:%M'),
    },
    {
        order: 2,
        key: TimeGranularityKey.HOUR,
        interval: timeHour,
        // format: (d: Date) => d.toLocaleString(preferredLocale, { hour: 'numeric', minute: '2-digit' }),
        format: formatHour,
        formatFull: timeFormat('%a %d %b %Y, %I %p'),
    },
    {
        order: 3,
        key: TimeGranularityKey.DAY,
        interval: timeDay,
        format: formatDay,
        formatFull: timeFormat('%a %d %b %Y'),
    },
    {
        order: 4,
        key: TimeGranularityKey.MONTH,
        interval: timeMonth,
        format: formatMonth,
        formatFull: timeFormat('%b %Y'),
    },
    {
        order: 5,
        key: TimeGranularityKey.YEAR,
        interval: timeYear,
        format: formatYear,
        formatFull: timeFormat('%Y'),
    },
];
export function getTimeGranularityByOrder(order) {
    return TIME_GRANULARITIES.find(function (s) { return s.order === order; });
}
export function getTimeGranularityForDate(date) {
    var e_2, _a;
    var prev = undefined;
    try {
        for (var TIME_GRANULARITIES_1 = __values(TIME_GRANULARITIES), TIME_GRANULARITIES_1_1 = TIME_GRANULARITIES_1.next(); !TIME_GRANULARITIES_1_1.done; TIME_GRANULARITIES_1_1 = TIME_GRANULARITIES_1.next()) {
            var current = TIME_GRANULARITIES_1_1.value;
            var interval = current.interval;
            var floored = interval(date);
            if (floored < date) {
                if (!prev)
                    return current;
                return prev;
            }
            prev = current;
        }
    }
    catch (e_2_1) { e_2 = { error: e_2_1 }; }
    finally {
        try {
            if (TIME_GRANULARITIES_1_1 && !TIME_GRANULARITIES_1_1.done && (_a = TIME_GRANULARITIES_1.return)) _a.call(TIME_GRANULARITIES_1);
        }
        finally { if (e_2) throw e_2.error; }
    }
    return TIME_GRANULARITIES[TIME_GRANULARITIES.length - 1];
}
export function areRangesEqual(a, b) {
    if (!a && !b)
        return true;
    if (!a || !b)
        return false;
    return a[0] === b[0] && a[1] === b[1];
}
//# sourceMappingURL=time.js.map