import * as Cluster from '@flowmap.gl/cluster';
export declare enum ConfigPropName {
    TITLE = "title",
    DESCRIPTION = "description",
    AUTHOR_NAME = "createdBy.name",
    AUTHOR_URL = "createdBy.url",
    SOURCE_NAME = "source.name",
    SOURCE_URL = "source.url",
    MAP_BBOX = "map.bbox",
    IGNORE_ERRORS = "ignore.errors",
    MAPBOX_ACCESS_TOKEN = "mapbox.accessToken",
    MAPBOX_MAP_STYLE = "mapbox.mapStyle",
    COLORS_SCHEME = "colors.scheme",
    COLORS_DARK_MODE = "colors.darkMode",
    ANIMATE_FLOWS = "animate.flows",
    FADE_AMOUNT = "fadeAmount",
    BASE_MAP_OPACITY = "baseMapOpacity",
    CLUSTER_ON_ZOOM = "clustering",
    FLOWS_SHEETS = "flows.sheets"
}
export interface ConfigProp {
    property: ConfigPropName;
    value: string | undefined;
}
export declare type Config = Record<string | ConfigPropName, string | undefined>;
export declare const getFlowTime: (flow: Flow) => Date | undefined;
export declare const getFlowMagnitude: (flow: Flow) => number;
export declare const getFlowOriginId: (flow: Flow) => string;
export declare const getFlowDestId: (flow: Flow) => string;
export declare const getLocationId: (loc: Location) => string;
export declare const getLocationCentroid: (location: Location | Cluster.Cluster) => [number, number];
export interface Location {
    id: string;
    lon: number;
    lat: number;
    name: string;
}
export interface LocationTotals {
    incoming: number;
    outgoing: number;
    within: number;
}
export declare function isLocationCluster(l: Location | Cluster.Cluster): l is Cluster.Cluster;
export interface Flow {
    origin: string;
    dest: string;
    count: number;
    time?: Date;
}
export interface CountByTime {
    time: Date;
    count: number;
}
export interface ViewportProps {
    width: number;
    height: number;
    latitude: number;
    longitude: number;
    zoom: number;
    bearing: number;
    pitch: number;
    altitude: number;
    maxZoom: number;
    minZoom: number;
    maxPitch: number;
    minPitch: number;
    transitionDuration?: number | 'auto';
    transitionInterpolator?: any;
    transitionInterruption?: any;
    transitionEasing?: any;
}
export declare type AsyncState<T> = {
    loading: boolean;
    error?: undefined;
    value?: undefined;
} | {
    loading?: false;
    error: Error;
    value?: undefined;
} | {
    loading?: false;
    error?: undefined;
    value: T;
};
//# sourceMappingURL=types.d.ts.map